import camelCase from 'camelcase'
import fs from 'fs/promises'
import { join } from 'path'
// import pluralize from 'pluralize'
import semver from 'semver'
import yaml from 'yaml'

type InputItem = {
  path: string
  title: string | undefined
  parts: string[]
}

type NestedItem = {
  text?: string
  link?: string
  items?: NestedItem[]
  collapsed?: boolean
}

type NestedStructure = {
  text: string
  items: NestedItem[]
}

const packageJsonPath = join(__dirname, '..', 'package.json')
const tmpRepoPath = join(__dirname, '..', 'temp_repo')
const lastStableVersionPath = join(tmpRepoPath, 'stable.version')
const lastUnstableVersionPath = join(tmpRepoPath, 'unstable.version')
const indexMdPath = join(__dirname, '..', 'vitepress', 'index.md')
const versionsMdPath = join(__dirname, '..', 'vitepress', 'versions.md')
const vitePressConfigPath = join(__dirname, '..', 'vitepress', '.vitepress', 'config.mts')

const getVersionFromPath = async (path: string): Promise<string | undefined> => {
  try {
    const contents = await fs.readFile(path, 'utf8')
    return contents.trim()
  } catch {
    return undefined
  }
}

const listAllMarkdownDocuments = async (path: string): Promise<string[]> => {
  let files: string[] = []
  const items = await fs.readdir(path, { withFileTypes: true })
  for (const item of items) {
    const fullPath = join(path, item.name)
    if (item.isDirectory()) {
      files = files.concat(await listAllMarkdownDocuments(fullPath))
    } else if (item.name.endsWith('.md')) {
      files.push(fullPath)
    }
  }
  return files
}

const listAllFilesWithInfo = async (path: string, pkg: any) => {
  const files = await listAllMarkdownDocuments(path)
  return await Promise.all(
    files.sort(sortForFilePaths).map(async (file) => {
      const raw = await fs.readFile(file, 'utf8')
      const h1s = raw
        .split('\n')
        .filter((line) => line.startsWith('# '))
        .map((line) => line.replace(/^#+\s*/, ''))
      const f = file.replace(path, '').replace(/\.md$/, '')
      const linkParts = f
        .split('/')
        .filter((link) => 'string' === typeof link && link.trim().length > 0)
      let firstTitle = h1s.shift()
      let replaceLastPart = false
      if (firstTitle && firstTitle.includes(': ')) {
        firstTitle = firstTitle.split(': ').pop()
        replaceLastPart = true
      }
      if (firstTitle && firstTitle.includes('/') && firstTitle !== pkg.name) {
        firstTitle = firstTitle.split('/').pop()
        replaceLastPart = true
      }
      const lastLinkPart = linkParts[linkParts.length - 1]
      const lastLinkPartDotCount = lastLinkPart.split('.').length - 1
      if (lastLinkPartDotCount > 1 && replaceLastPart === true) {
        replaceLastPart = true
        const lastLinkPartParts = lastLinkPart.split('.')
        lastLinkPartParts.shift()
        // replace the last part with the first title
        lastLinkPartParts.pop()
        lastLinkPartParts.push(firstTitle!)
        firstTitle = lastLinkPartParts.join('.')
      }
      const fixedLinkParts: Array<string> = linkParts
        .map((part, i) => {
          if (part === 'index' && i < 1) {
            return 'Getting Started'
          } else if (part === 'index' && i >= 1) {
            return 'Default'
          } else if (replaceLastPart && i === linkParts.length - 1) {
            return firstTitle
          } else if (i < 1) {
            return camelCase(part, { pascalCase: true })
          } else {
            return part
          }
        })
        .filter((p) => 'string' === typeof p && p.trim().length > 0) as Array<string>
      if ('string' === typeof firstTitle) {
        firstTitle = firstTitle.replace(/\\<.*\\>/g, '')
      }
      return {
        path: f,
        title: firstTitle,
        parts: fixedLinkParts,
      }
    })
  )
}

function createSidebar(title: string, array: InputItem[], baseDir?: string): NestedStructure[] {
  const sidebar: NestedStructure = { text: title, items: [] }
  const categories: Record<string, NestedItem> = {}

  array.forEach((item) => {
    const category = item.parts[0]
    const text = item.title
    const link = baseDir ? `${baseDir}${item.path}` : item.path

    if (!categories[category]) {
      categories[category] = { text: category, collapsed: true, items: [] as NestedItem[] }
      sidebar.items.push(categories[category])
    }

    categories[category].items?.push({ text, link })
  })
  sidebar.items = sidebar.items.map((cat) => {
    if (cat.items && cat.items.length === 1) {
      return {
        text: cat.text,
        link: cat.items[0].link,
      }
    }
    return cat
  })
  sidebar.items = sidebar.items.sort((a, b) => {
    if (a.text === 'Getting Started') {
      return -1
    }
    if (b.text === 'Getting Started') {
      return 1
    }
    if (!a.text || !b.text) {
      return 0
    }
    return a.text.localeCompare(b.text)
  })
  return [sidebar]
}

const sortForFilePaths = (a: string, b: string): number => {
  if (a === b) {
    return 0
  }
  if (a === 'index.md') {
    return -1
  }
  if (b === 'index.md') {
    return 1
  }
  const aParts = a.split('/')
  const bParts = b.split('/')
  if (aParts.length !== bParts.length) {
    return aParts.length - bParts.length
  }
  const aLastPart = aParts.pop()
  const bLastPart = bParts.pop()
  return aLastPart!.localeCompare(bLastPart!)
}

const sortForUnstableVersions = (latest: string | undefined, a: string, b: string): number => {
  if (a === b) {
    return 0
  }
  if (a === latest) {
    return -1
  }
  if (b === latest) {
    return 1
  }
  const aParts = a.split('-')
  const bParts = b.split('-')
  if (aParts.length !== bParts.length) {
    return aParts.length - bParts.length
  }
  const aLastPart = aParts.pop()
  const bLastPart = bParts.pop()
  return aLastPart!.localeCompare(bLastPart!)
}

Promise.all([
  getVersionFromPath(lastStableVersionPath),
  getVersionFromPath(lastUnstableVersionPath),
  fs.readdir(tmpRepoPath),
  fs.readFile(packageJsonPath, 'utf8'),
]).then(async ([lastStableVersion, lastUnstableVersion, files, pkgJson]) => {
  const pkg = JSON.parse(pkgJson)
  const tagged = new Set<string>()
  const fromBranches = new Set<string>()
  if ('string' === typeof lastStableVersion) {
    tagged.add(lastStableVersion)
  }
  if ('string' === typeof lastUnstableVersion) {
    fromBranches.add(lastUnstableVersion)
  }
  files.forEach((file) => {
    if (file.endsWith('.md') || file.endsWith('.version') || !semver.valid(file)) {
      // do nothing
    } else {
      if (file.match(/v?\d+\.\d+\.\d+-[a-zA-Z0-9]+-[a-zA-Z0-9]{8}/gm)) {
        fromBranches.add(file)
      } else {
        tagged.add(file)
      }
    }
  })
  const lastStableVersionFromDirectories = Array.from(tagged).sort(semver.rcompare).shift()
  const indexManifest: any = {
    layout: 'home',
    hero: {
      name: 'DocFox SDK',
      tagline: 'A NodeJS SDK based on the DocFox API Documentation',
      actions: [
        {
          theme: 'brand',
          text: 'Latest Stable Version',
          link: lastStableVersionFromDirectories
            ? `/${lastStableVersionFromDirectories}/`
            : undefined,
        },
        {
          theme: 'brand',
          text: 'Latest Development Version',
          link: lastUnstableVersion ? `/${lastUnstableVersion}/` : undefined,
        },
        {
          theme: 'alt',
          text: 'All Versions',
          link: '/versions',
        },
        {
          theme: 'alt',
          text: 'DocFox API Documentation',
          link: 'https://www.docfoxapp.com/api/v2/documentation',
        },
      ].filter((a) => a.link),
    },
  }
  const indexMdContent = `---
${yaml.stringify(indexManifest)}
---`
  await fs.writeFile(indexMdPath, indexMdContent)

  const versionsMdContent = `# Documented Versions

  ## Stable Versions

  ${Array.from(tagged)
    .map((v) => `- [${v}](/${v}/)`)
    .sort(semver.rcompare)
    .join('\n')}

  ## Development Versions

  ${Array.from(fromBranches)
    .map((v) => `- [${v}](/${v}/)`)
    .sort(sortForUnstableVersions.bind(null, lastUnstableVersion))
    .join('\n')}
`

  await fs.writeFile(versionsMdPath, versionsMdContent)

  const sidebar = Object.assign(
    {},
    {
      '/versions': [
        {
          text: 'Versions',
          items: [
            {
              theme: 'brand',
              text: 'Latest Stable Version',
              link: lastStableVersionFromDirectories
                ? `/${lastStableVersionFromDirectories}/`
                : undefined,
            },
            {
              theme: 'brand',
              text: 'Latest Development Version',
              link: lastUnstableVersion ? `/${lastUnstableVersion}/` : undefined,
            },
          ].filter((a) => a.link),
        },
      ],
    },
    ...(await Promise.all(
      [...tagged, ...fromBranches].map(async (p) => {
        const files = await listAllFilesWithInfo(join(tmpRepoPath, p), pkg)
        const sidebar = createSidebar('SDK Documentation', files, `/${p}`)
        return {
          [`/${p}/`]: sidebar,
        }
      })
    ))
  )

  const pagesUrl = new URL(process.env.CI_PAGES_URL || 'http://localhost/')

  const vitePressConfig: any = {
    base: pagesUrl.pathname,
    title: 'Docfox SDK',
    description: 'Documentation for using the Docfox SDK',
    themeConfig: {
      nav: [
        { text: 'Home', link: '/' },
        {
          text: 'Stable',
          link: lastStableVersionFromDirectories
            ? `/${lastStableVersionFromDirectories}/`
            : undefined,
        },
        { text: 'Unstable', link: lastUnstableVersion ? `/${lastUnstableVersion}/` : undefined },
        { text: 'Versions', link: '/versions' },
      ].filter((a) => a.link),
      outline: [2, 3],
      sidebar,
      socialLinks: [
        {
          icon: {
            svg: '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 380 380"><defs><style>.cls-1{fill:#e24329;}.cls-2{fill:#fc6d26;}.cls-3{fill:#fca326;}</style></defs><g id="LOGO"><path class="cls-1" d="M282.83,170.73l-.27-.69-26.14-68.22a6.81,6.81,0,0,0-2.69-3.24,7,7,0,0,0-8,.43,7,7,0,0,0-2.32,3.52l-17.65,54H154.29l-17.65-54A6.86,6.86,0,0,0,134.32,99a7,7,0,0,0-8-.43,6.87,6.87,0,0,0-2.69,3.24L97.44,170l-.26.69a48.54,48.54,0,0,0,16.1,56.1l.09.07.24.17,39.82,29.82,19.7,14.91,12,9.06a8.07,8.07,0,0,0,9.76,0l12-9.06,19.7-14.91,40.06-30,.1-.08A48.56,48.56,0,0,0,282.83,170.73Z"/><path class="cls-2" d="M282.83,170.73l-.27-.69a88.3,88.3,0,0,0-35.15,15.8L190,229.25c19.55,14.79,36.57,27.64,36.57,27.64l40.06-30,.1-.08A48.56,48.56,0,0,0,282.83,170.73Z"/><path class="cls-3" d="M153.43,256.89l19.7,14.91,12,9.06a8.07,8.07,0,0,0,9.76,0l12-9.06,19.7-14.91S209.55,244,190,229.25C170.45,244,153.43,256.89,153.43,256.89Z"/><path class="cls-2" d="M132.58,185.84A88.19,88.19,0,0,0,97.44,170l-.26.69a48.54,48.54,0,0,0,16.1,56.1l.09.07.24.17,39.82,29.82s17-12.85,36.57-27.64Z"/></g><script xmlns=""/></svg>',
          },
          link: 'https://gitlab.jak.guru/public-libs/docfox/sdk',
        },
      ],
    },
  }

  const vitePressConfigContent = `import { defineConfig } from 'vitepress'
export default defineConfig(${JSON.stringify(vitePressConfig, null, 2)})`
  await fs.writeFile(vitePressConfigPath, vitePressConfigContent)
})
