import camelCase from 'camelcase'
import fs from 'fs/promises'
import { join } from 'path'
import yaml from 'yaml'

interface Param {
  name: string
  type: string
  optional?: boolean
  default?: string
}

interface Method {
  name: string
  params: Record<string, Param>
  payloadRequired?: boolean
  payload?: Record<string, Param>
}

const generateType = (params: Record<string, Param>): string => {
  const paramArray = Object.keys(params).map((paramName) => {
    const param = params[paramName]
    return `"${param.name}"${param.optional ? '?' : ''}: ${param.type}`
  })
  return `{${paramArray.join('; ')}}`
}

const operationToFunction = (operation: Method): string => {
  let ret = ''
  const argumentsArray = [`params: ${generateType(operation.params)}`]
  if (operation.payload) {
    argumentsArray.push(
      `payload${operation.payloadRequired ? '' : '?'}: ${generateType(operation.payload ?? {})}`
    )
  }
  ret += `public async ${operation.name}(${argumentsArray.join(', ')}): ReturnType<DocFoxClient['${
    operation.name
  }']> {\n`
  ret += `    return this.#doClientRequestWithAuthenticationRetry<Awaited<ReturnType<DocFoxClient['${
    operation.name
  }']>>>(this.#client.${operation.name}.bind(this.#client, params${
    operation.payload ? ', payload' : ''
  }))\n`
  ret += '}'
  return ret
}

const specPath = join(__dirname, '..', 'specs.yml')
fs.readFile(specPath, 'utf8').then(async (spec) => {
  const parsed = yaml.parse(spec)
  const operations = new Map<string, Method>()
  for (const path in parsed.paths) {
    for (const method in parsed.paths[path]) {
      const parameters = Object.assign(
        {},
        ...parsed.paths[path][method].parameters.map((p) => {
          const paramName = camelCase(p.name, {
            pascalCase: false,
            preserveConsecutiveUppercase: true,
          })
          const type = p.schema?.type ?? 'any'
          const optional = p.required !== true
          let defaultValue: string | undefined
          if ('string' === type) {
            if (p.schema && p.schema.enum) {
              defaultValue = p.schema.enum[0]
            }
          }
          const paramOptions: Param = {
            name: p.name,
            type,
            optional,
          }
          if (defaultValue) {
            paramOptions.default = defaultValue
          }
          return { [paramName]: paramOptions }
        })
      ) as Record<string, Param>
      const operation: Method = {
        name: parsed.paths[path][method].operationId,
        params: parameters,
      }
      if (parsed.paths[path][method].requestBody) {
        const contentType = Object.keys(parsed.paths[path][method].requestBody.content)[0]
        operation.payload = Object.assign(
          {},
          ...Object.keys(
            parsed.paths[path][method].requestBody.content[contentType].schema.properties
          ).map((p) => {
            const paramName = camelCase(p, {
              pascalCase: false,
              preserveConsecutiveUppercase: true,
            })
            const type =
              parsed.paths[path][method].requestBody.content[contentType].schema.properties[p].type
            const optional =
              !parsed.paths[path][method].requestBody.content[
                contentType
              ].schema.required?.includes(p)
            let defaultValue: string | undefined
            if ('string' === type) {
              if (
                parsed.paths[path][method].requestBody.content[contentType].schema.properties[p]
                  .enum
              ) {
                defaultValue =
                  parsed.paths[path][method].requestBody.content[contentType].schema.properties[p]
                    .enum[0]
              }
            }
            const paramOptions: Param = {
              name: p,
              type,
              optional,
            }
            if (defaultValue) {
              paramOptions.default = defaultValue
            }
            return { [paramName]: paramOptions }
          })
        ) as Record<string, Param>
      }
      operations.set(operation.name, operation)
    }
  }
  operations.forEach((operation) => {
    console.log(operationToFunction(operation))
    console.log('')
  })
})
