import camelCase from 'camelcase'
import fs from 'fs/promises'
import { join } from 'path'
// import semver from 'semver'
import dot from 'dot-object'
// import pluralize from 'pluralize'
import { exec as execCallback } from 'child_process'
import { inspect, promisify } from 'util'
import yaml from 'yaml'
const exec = promisify(execCallback)

dot.keepArray = true

interface Param {
  name: string
  type: string
  optional?: boolean
  default?: string
}

interface ResponseParam {
  name: string
  type: string | Record<string, ResponseParam>
  optional?: boolean
}

interface ResponseSchema {
  headers: Record<string, ResponseParam>
  content?: Record<string, ResponseParam> | undefined
}

interface Method {
  name: string
  params: Record<string, Param>
  headers: Record<string, Param>
  payloadRequired?: boolean
  payload?: Record<string, Param>
  path: string
  method: string
  tag: string
  responses: Record<string, ResponseSchema>
}

const baseDir = join(__dirname, '..')
const specPath = join(baseDir, 'specs.yml')

async function walk(
  item: any,
  key: string | number | undefined,
  parent: any | undefined,
  callback: (
    item: any,
    key?: string | number | undefined,
    parent?: any | undefined,
    keyOfArray?: string | number | undefined
  ) => any | Promise<any>,
  keyOfArray?: string | number | undefined
) {
  if (Array.isArray(item)) {
    item = await Promise.all(item.map(async (i, k) => await walk(i, k, parent, callback, key)))
  } else if ('object' === typeof item && null !== item) {
    item = Object.assign(
      {},
      ...(await Promise.all(
        Object.keys(item).map(async (key) => {
          return { [key]: await walk(item[key], key, parent, callback) }
        })
      ))
    )
  }
  return await callback(item, key, parent, keyOfArray)
}

const getObjectTypeOf = (obj: any): string => {
  switch (typeof obj) {
    case 'object':
      if (null === obj) {
        return 'null'
      }
      if (Array.isArray(obj)) {
        return 'array'
      }
      return 'object'
    default:
      return typeof obj
  }
}

const makeResponseTypeSchema = async (schema: any): Promise<ResponseSchema> => {
  const res: ResponseSchema = {
    headers: {},
  }
  if (schema.headers) {
    for (const headerName in schema.headers) {
      res.headers[headerName] = {
        name: headerName,
        type: 'string',
        optional: schema.headers[headerName].required === false,
      } as ResponseParam
    }
  }
  if (schema.content) {
    res.content = {}
    for (const contentType in schema.content) {
      const { schema: s, example } = schema.content[contentType]
      let fromSchema = await walk(s, undefined, undefined, (item, key) => {
        if ('object' === typeof item && null !== item) {
          if (item.properties) {
            return item.properties
          } else if (item.type) {
            item.type = {
              name: key,
              type: getTsTypeForSchemaType(item.type),
              optional: item.nullable === true,
            }
          }
          const props = Object.keys(item).filter(
            (k) => !['type', 'properties', 'description'].includes(k)
          )
          if (props.length === 0) {
            return item.type
          }
        }
        return item
      })
      if (!fromSchema) {
        fromSchema = {}
      }
      fromSchema = await walk(fromSchema, undefined, undefined, (item) => {
        if ('object' === typeof item && null !== item) {
          if (item.type) {
            if (item.type.type && item.type.type.type) {
              item.type = item.type.type
            }
          }
        }
        return item
      })
      for (const key in example) {
        const tsType = getTsTypeForSchemaType(getObjectTypeOf(example[key]))
        if (fromSchema[key]) {
          const schemaType = getTsTypeForSchemaType(getObjectTypeOf(fromSchema[key]))
          if (schemaType !== tsType) {
            if ('any[]' === tsType) {
              if (
                'object' === typeof fromSchema[key] &&
                'string' === typeof fromSchema[key].type &&
                'string' === typeof fromSchema[key].name
              ) {
                fromSchema[key] = `${fromSchema[key].type}[]`
              } else {
                const interfaced = await walk(fromSchema[key], undefined, undefined, (item) => {
                  if (
                    'object' === typeof item &&
                    null !== item &&
                    'string' === typeof item.type &&
                    'string' === typeof item.name
                  ) {
                    return `${item.type}`
                  } else if ('object' === typeof item && null !== item) {
                    return generateInterface(item)
                  }
                  return item
                })
                fromSchema[key] = {
                  name: key,
                  type: `Array<${interfaced}>`,
                  optional: false,
                }
              }
            } else {
              console.log(key, schemaType, tsType)
              console.log(inspect(fromSchema[key], false, 10, false))
            }
          }
        } else {
          fromSchema[key] = {
            name: key,
            type: tsType,
            optional: false,
          }
        }
      }
      if ('object' === typeof fromSchema && null !== fromSchema && !Array.isArray(fromSchema)) {
        if (
          undefined === fromSchema.name &&
          'undefined' !== typeof fromSchema.type &&
          'undefined' !== typeof fromSchema.optional
        ) {
          if (fromSchema.type === 'Record<string, any>') {
            const objectWithoutTypeInformation = Object.assign(
              {},
              ...Object.keys(fromSchema)
                .filter((k) => !['type', 'name', 'optional'].includes(k))
                .map((k) => {
                  return { [k]: fromSchema[k] }
                })
            )
            fromSchema = objectWithoutTypeInformation
          } else {
            fromSchema = {}
          }
        }
      }
      res.content = Object.assign({}, res.content, fromSchema)
    }
  }
  return res
}

const generateType = (params: Record<string, Param>, onEmpty: string = '{}'): string => {
  const paramArray = Object.keys(params)
    .filter((paramName) => 'string' === typeof params[paramName].type)
    .map((paramName) => {
      const param = params[paramName]
      return `"${param.name}"${param.optional ? '?' : ''}: ${param.type}`
    })
  if (paramArray.length === 0) {
    return onEmpty
  }
  return `{${paramArray.join('; ')}}`
}

const generateInterface = (params: Record<string, string>): string => {
  const paramArray = Object.keys(params).map((paramName) => {
    const param = params[paramName]
    return `"${paramName}": ${param}`
  })
  return `{${paramArray.join('; ')}}`
}

const fixPathToUseCamelCasedParams = (path: string): string => {
  const pathParams = path.match(/\{([^\}]+)\}/g)
  if (pathParams) {
    pathParams.forEach((pathParam) => {
      const paramName = pathParam.replace(/\{|\}/g, '')
      const camelCasedParamName = camelCase(paramName, {
        pascalCase: false,
        preserveConsecutiveUppercase: true,
      })
      path = path.replace(pathParam, `{${camelCasedParamName}}`)
    })
  }
  return path
}

const operationToFunction = (operation: Method): string => {
  let docUri = 'https://www.docfoxapp.com/api/v2/documentation#tag/'
  docUri += operation.tag.replace(/\s/g, '-')
  docUri += '/paths/'
  docUri += encodeURIComponent(operation.path.replace(/\//g, '~1'))
  docUri += '/' + operation.method.toLowerCase()
  let ret = ''
  ret += '/**' + '\n'
  ret += ` * Operation Method for \`${operation.method}\` \`${operation.path}\`` + '\n'
  ret += ` * @tag ${operation.tag}` + '\n'
  ret += ` * @see [DocFox Documentation](${docUri})` + '\n'
  ret += ' */' + '\n'
  const argumentsArray = [`params: ${generateType(operation.params)}`]
  if (operation.payload) {
    argumentsArray.push(
      `payload${operation.payloadRequired ? '' : '?'}: ${generateType(operation.payload ?? {})}`
    )
  }
  argumentsArray.push('config?: AxiosRequestConfig<any>')
  ret += `public async ${operation.name}(${argumentsArray.join(
    ', '
  )}): ReturnType<DocFoxAxiosClient['${operation.name}']> {\n`
  ret += `    return await this.#doClientRequestWithAuthenticationRetry<Awaited<ReturnType<DocFoxAxiosClient['${
    operation.name
  }']>>>(this.#client.${operation.name}.bind(this.#client, params${
    operation.payload ? ', payload' : ', undefined'
  }, config))\n`
  ret += '}'
  return ret
}

const getTsTypeForSchemaType = (type: string): string => {
  switch (type) {
    case 'string':
      return 'string'
    case 'integer':
      return 'number'
    case 'boolean':
      return 'boolean'
    case 'array':
      return 'any[]'
    case 'object':
      return 'Record<string, any>'
    case null:
      return 'any'
    case undefined:
      return 'any'
    default:
      return type
  }
}

fs.readFile(specPath, 'utf8').then(async (spec) => {
  const parsed = yaml.parse(spec)
  // write out the new definitions object as a json object
  console.log(
    `Writing definitions to ${join(baseDir, 'definitions.tmp.ts')}. This will take a bit of time.`
  )
  await fs.writeFile(
    join(baseDir, 'definitions.tmp.ts'),
    `import type { OpenAPIV3 } from 'openapi-types'

  export const definitionDocument: OpenAPIV3.Document = ${JSON.stringify(parsed, null, 2)}`
  )
  await exec('eslint definitions.tmp.ts --ext=.ts --fix --no-ignore')
  console.log(`Wrote defintitions to ${join(baseDir, 'definitions.tmp.ts')}`)
  // create the endpoint methods with types and typedoc comments from the definitions
  const operations = new Map<string, Method>()
  for (const path in parsed.paths) {
    for (const method in parsed.paths[path]) {
      const parameters = Object.assign(
        {},
        ...parsed.paths[path][method].parameters.map((p) => {
          const paramName = camelCase(p.name, {
            pascalCase: false,
            preserveConsecutiveUppercase: true,
          })
          const type = p.schema?.type ?? 'any'
          const optional = p.required !== true
          let defaultValue: string | undefined
          if ('string' === type) {
            if (p.schema && p.schema.enum) {
              defaultValue = p.schema.enum[0]
            }
          }
          const paramOptions: Param = {
            name: p.name,
            type: getTsTypeForSchemaType(type),
            optional,
          }
          if (defaultValue) {
            paramOptions.default = defaultValue
          }
          return { [paramName]: paramOptions }
        })
      ) as Record<string, Param>
      const headers = Object.assign(
        {},
        ...parsed.paths[path][method].parameters
          .filter((p) => p.in === 'header')
          .map((p) => {
            const paramName = camelCase(p.name, {
              pascalCase: false,
              preserveConsecutiveUppercase: true,
            })
            const type = p.schema?.type ?? 'any'
            const optional = p.required !== true
            let defaultValue: string | undefined
            if ('string' === type) {
              if (p.schema && p.schema.enum) {
                defaultValue = p.schema.enum[0]
              }
            }
            const paramOptions: Param = {
              name: p.name,
              type: getTsTypeForSchemaType(type),
              optional,
            }
            if (defaultValue) {
              paramOptions.default = defaultValue
            }
            return { [paramName]: paramOptions }
          })
      ) as Record<string, Param>
      const responses: Record<string, ResponseSchema> = {}
      // lets make a response
      for (const responseCode in parsed.paths[path][method].responses) {
        const response = parsed.paths[path][method].responses[responseCode]
        responses[responseCode] = await makeResponseTypeSchema(response)
      }
      const operation: Method = {
        name: parsed.paths[path][method].operationId,
        params: parameters,
        headers,
        path,
        method: method.toUpperCase(),
        tag: parsed.paths[path][method].tags[0],
        responses,
      }
      if (parsed.paths[path][method].requestBody) {
        const contentType = Object.keys(parsed.paths[path][method].requestBody.content)[0]
        operation.payload = Object.assign(
          {},
          ...Object.keys(
            parsed.paths[path][method].requestBody.content[contentType].schema.properties
          ).map((p) => {
            const paramName = camelCase(p, {
              pascalCase: false,
              preserveConsecutiveUppercase: true,
            })
            const type = getTsTypeForSchemaType(
              parsed.paths[path][method].requestBody.content[contentType].schema.properties[p].type
            )
            const optional =
              !parsed.paths[path][method].requestBody.content[
                contentType
              ].schema.required?.includes(p)
            let defaultValue: string | undefined
            if ('string' === type) {
              if (
                parsed.paths[path][method].requestBody.content[contentType].schema.properties[p]
                  .enum
              ) {
                defaultValue =
                  parsed.paths[path][method].requestBody.content[contentType].schema.properties[p]
                    .enum[0]
              }
            }
            const paramOptions: Param = {
              name: p,
              type,
              optional,
            }
            if (defaultValue) {
              paramOptions.default = defaultValue
            }
            return { [paramName]: paramOptions }
          })
        ) as Record<string, Param>
      }
      delete operation.params.authorization
      delete operation.params.accept
      delete operation.params.contentType
      operations.set(operation.name, operation)
    }
  }
  let endpointMethods = ''
  operations.forEach((operation) => {
    endpointMethods += `${operationToFunction(operation)}\n\n`
  })
  await fs.writeFile(join(baseDir, 'methods.tmp'), endpointMethods)
  console.log(`Wrote methods to ${join(baseDir, 'methods.tmp')}`)
  // create the client types
  let clienttypes = ''
  if (parsed.components) {
    clienttypes += `export declare namespace Components {\n`
    for (const componentName in parsed.components) {
      const componentTypeName = camelCase(componentName, { pascalCase: true })
      let component = await walk(
        parsed.components[componentName],
        undefined,
        undefined,
        (item, key) => {
          if ('object' === typeof item && null !== item) {
            if (item.properties) {
              return item.properties
            } else if (item.type) {
              item.type = {
                name: key,
                type: getTsTypeForSchemaType(item.type),
                optional: item.nullable === true,
              }
            }
            const props = Object.keys(item).filter(
              (k) =>
                ![
                  'type',
                  'properties',
                  'minLength',
                  'maxLength',
                  'title',
                  'format',
                  'description',
                ].includes(k)
            )
            if (props.length === 0) {
              return item.type
            }
          }
          return item
        }
      )
      component = await walk(component, undefined, undefined, (item) => {
        if (
          'object' === typeof item &&
          null !== item &&
          'string' === typeof item.type &&
          'string' === typeof item.name &&
          'boolean' === typeof item.optional
        ) {
          return item.type
        }
        return item
      })
      clienttypes += `  namespace ${componentTypeName} {\n`
      for (const t in component) {
        const typeName = camelCase(t, { pascalCase: true })
        if ('object' === typeof component[t] && null !== component[t]) {
          clienttypes += `    export interface ${typeName} {\n`
          Object.keys(component[t]).forEach((k) => {
            clienttypes += `      ${k}: ${component[t][k]}\n`
          })
          clienttypes += `    }\n`
        } else {
          clienttypes += `    export type ${typeName} = ${component[t]}\n`
        }
      }
      clienttypes += `  }\n`
    }
    clienttypes += `}\n`
    clienttypes += '\n'
  }
  clienttypes += `export declare namespace Paths {\n`
  const ops = Array.from(operations.values())
  for (let i = 0; i < ops.length; i++) {
    const operation = ops[i]
    const namespaceName = camelCase(operation.name, { pascalCase: true })
    clienttypes += `  namespace ${namespaceName} {\n`
    clienttypes += `    export interface HeaderParameters {\n`
    Object.keys(operation.headers).forEach((headerName) => {
      if (operation.params[headerName]) {
        const headerTypeName = camelCase(headerName, { pascalCase: true })
        clienttypes += `      ${JSON.stringify(
          operation.headers[headerName].name
        )}: Parameters.${headerTypeName}\n`
      }
    })
    clienttypes += `    }\n`
    clienttypes += `    export interface PathParameters {\n`
    Object.keys(operation.params).forEach((paramName) => {
      if (fixPathToUseCamelCasedParams(operation.path).includes(`{${paramName}}`)) {
        const pathParamTypeName = camelCase(paramName, { pascalCase: true })
        clienttypes += `      ${JSON.stringify(
          operation.params[paramName].name
        )}: Parameters.${pathParamTypeName}\n`
      }
    })
    clienttypes += `    }\n`
    clienttypes += `    export interface QueryParameters {\n`
    Object.keys(operation.params).forEach((paramName) => {
      if (
        !fixPathToUseCamelCasedParams(operation.path).includes(`{${paramName}}`) &&
        !operation.headers[paramName]
      ) {
        const pathParamTypeName = camelCase(paramName, { pascalCase: true })
        clienttypes += `      ${JSON.stringify(
          operation.params[paramName].name
        )}: Parameters.${pathParamTypeName}\n`
      }
    })
    clienttypes += `    }\n`
    clienttypes += `    export interface OperationParameters {\n`
    Object.keys(operation.params).forEach((paramName) => {
      const pathParamTypeName = camelCase(paramName, { pascalCase: true })
      clienttypes += `      ${JSON.stringify(
        operation.params[paramName].name
      )}: Parameters.${pathParamTypeName}\n`
    })
    clienttypes += `    }\n`
    clienttypes += `    namespace Parameters {\n`
    Object.keys(operation.params).forEach((paramName) => {
      const param = operation.params[paramName]
      const paramTypeName = camelCase(paramName, { pascalCase: true })
      clienttypes += `      export type ${paramTypeName} = ${param.type}\n`
    })
    clienttypes += `    }\n`
    clienttypes += `    namespace Responses {\n`
    for (const code in operation.responses) {
      clienttypes += `      export interface $${code} {\n`
      const typedOperation = await walk(
        operation.responses[code].content,
        undefined,
        undefined,
        (item, key) => {
          if ('object' === typeof item && null !== item) {
            if ('string' === typeof item.type && 'string' === typeof item.name) {
              return item.type
            } else if ('undefined' !== typeof key) {
              return generateInterface(item)
            }
          }
          return item
        }
      )
      for (const tok in typedOperation) {
        clienttypes += `        ${tok}: ${typedOperation[tok]}\n`
      }
      clienttypes += `      }\n`
    }
    clienttypes += `    }\n`
    clienttypes += `  }\n`
  }
  clienttypes += `}\n`
  clienttypes += '\n'
  clienttypes += `export interface OperationMethods {\n`
  for (let i = 0; i < ops.length; i++) {
    const operation = ops[i]
    const namespaceName = camelCase(operation.name, { pascalCase: true })
    const responseCode = Object.keys(operation.responses)
      .filter((c) => parseInt(c) >= 200 && parseInt(c) < 400)
      .sort((a, b) => {
        return parseInt(a) - parseInt(b)
      })
      .shift()
    if (responseCode) {
      const responseCodes = Object.keys(operation.responses)
      const operationResponses = responseCodes.map((code) => {
        return `Paths.${namespaceName}.Responses.$${code}`
      })
      clienttypes += `  '${operation.name}'(\n`
      clienttypes += `    parameters?: Parameters<Paths.${namespaceName}.OperationParameters> | null | undefined,\n`
      clienttypes += `    payload?: ${(generateType(operation.payload ?? {}), 'any')},\n`
      clienttypes += `    config?: AxiosRequestConfig\n`
      clienttypes += `  ): OperationResponse<${operationResponses.join(' | ')}>\n`
      clienttypes += '\n'
    }
  }
  clienttypes += `}\n`
  clienttypes += '\n'
  clienttypes += `export interface OperationMethodResponses {\n`
  for (let i = 0; i < ops.length; i++) {
    const operation = ops[i]
    const namespaceName = camelCase(operation.name, { pascalCase: true })
    const responseCode = Object.keys(operation.responses)
      .filter((c) => parseInt(c) >= 200 && parseInt(c) < 400)
      .sort((a, b) => {
        return parseInt(a) - parseInt(b)
      })
      .shift()
    if (responseCode) {
      const responseCodes = Object.keys(operation.responses)
      const operationResponses = responseCodes.map((code) => {
        return `$${code}: Paths.${namespaceName}.Responses.$${code}`
      })
      clienttypes += `  '${operation.name}': {\n`
      for (let oi = 0; oi < operationResponses.length; oi++) {
        const res = operationResponses[oi]
        clienttypes += `    ${res}\n`
      }
      clienttypes += `  }\n`
      clienttypes += '\n'
    }
  }
  clienttypes += `}\n`
  clienttypes += '\n'
  const operationsByPathAndMethod: any = {}
  operations.forEach((operation) => {
    if (!operationsByPathAndMethod[operation.path]) {
      operationsByPathAndMethod[operation.path] = {}
    }
    operationsByPathAndMethod[operation.path][operation.method.toLowerCase()] = operation
  })
  clienttypes += `export interface PathsDictionary {\n`
  for (const path in operationsByPathAndMethod) {
    clienttypes += `  ['${path}']: {\n`
    for (const method in operationsByPathAndMethod[path]) {
      const operation = operationsByPathAndMethod[path][method]
      const namespaceName = camelCase(operation.name, { pascalCase: true })
      const responseCode = Object.keys(operation.responses)
        .filter((c) => parseInt(c) >= 200 && parseInt(c) < 400)
        .sort((a, b) => {
          return parseInt(a) - parseInt(b)
        })
        .shift()
      if (responseCode) {
        const responseCodes = Object.keys(operation.responses)
        const operationResponses = responseCodes.map((code) => {
          return `Paths.${namespaceName}.Responses.$${code}`
        })
        clienttypes += `    '${method}'(\n`
        clienttypes += `      parameters?: Parameters<Paths.${namespaceName}.OperationParameters> | null | undefined,\n`
        clienttypes += `      payload?: ${(generateType(operation.payload ?? {}), 'any')},\n`
        clienttypes += `      config?: AxiosRequestConfig\n`
        clienttypes += `    ): OperationResponse<${operationResponses.join(' | ')}>\n`
        clienttypes += '\n'
      }
    }
    clienttypes += `  }\n`
  }
  clienttypes += `}\n`
  clienttypes += '\n'
  clienttypes += `export type Client = OpenAPIClient<OperationMethods, PathsDictionary>\n`
  const clientTypeTemplateHeader = await fs.readFile(
    join(baseDir, 'templates', 'ClientTypes.ts.txt'),
    'utf8'
  )
  clienttypes = clientTypeTemplateHeader + '\n\n' + clienttypes
  await fs.writeFile(join(baseDir, 'clientTypes.tmp'), clienttypes)
  console.log(`Wrote client types to ${join(baseDir, 'clientTypes.tmp')}`)
})
