import fs from 'fs/promises'
import { join } from 'path'

const baseDir = join(__dirname, '..')
const docsFolderPath = join(baseDir, 'docs')
const coverageFolderPath = join(baseDir, 'coverage')
const coverageReportPath = join(coverageFolderPath, 'coverage-final.json')
const coverageSummaryPath = join(coverageFolderPath, 'coverage-summary.json')

const getCoveragePercentage = (coverage: any) => {
  const { lines, functions, branches, statements } = coverage
  const total = lines.total + functions.total + branches.total + statements.total
  const covered = lines.covered + functions.covered + branches.covered + statements.covered
  return (covered / total) * 100
}

const getColorIconForPercentage = (percentage: number) => {
  if (percentage >= 90) return '🟢'
  if (percentage >= 70) return '🟡'
  if (percentage >= 50) return '🟠'
  return '🔴'
}

const getMarkdownProgressBar = (percentage: number) => {
  if (!percentage) {
    return ''
  }
  percentage = parseFloat(percentage.toFixed(2))
  const colorIcon = getColorIconForPercentage(percentage)
  const dotsToMake = Math.floor(percentage / 10)
  let dots = '█'.repeat(dotsToMake)
  dots += '░'.repeat(10 - dotsToMake)
  return `${percentage.toFixed(2)}% ${colorIcon} [${dots}]`
}

Promise.all([fs.readFile(coverageReportPath), fs.readFile(coverageSummaryPath)]).then(
  async ([_rawReport, rawSummary]) => {
    const summary = JSON.parse(rawSummary.toString())
    let markdown = `# Testing Coverage Report\n\n`
    markdown += `## Summary\n\n`
    markdown += `| **File** | | **Statements** | | **Branches** | | **Functions** | | **Lines** | |
| --- | --- | --- | --- | --- | --- | --- | --- | --- | --- |
`
    Object.keys(summary).forEach((key) => {
      const coverage = summary[key]
      const file = key.includes(baseDir) ? key.replace(baseDir, '') : 'All Files'
      const percentage = getCoveragePercentage(coverage)
      const fileProgressBar = getMarkdownProgressBar(percentage)
      const parts = [
        file,
        fileProgressBar,
        `${coverage.statements.covered}/${coverage.statements.total}`,
        getMarkdownProgressBar(coverage.statements.pct),
        `${coverage.branches.covered}/${coverage.branches.total}`,
        getMarkdownProgressBar(coverage.branches.pct),
        `${coverage.functions.covered}/${coverage.functions.total}`,
        getMarkdownProgressBar(coverage.functions.pct),
        `${coverage.lines.covered}/${coverage.lines.total}`,
        getMarkdownProgressBar(coverage.lines.pct),
      ]
      markdown += `| ${parts.join(' | ')} |\n`
    })
    markdown += `\n`
    await fs.writeFile(join(docsFolderPath, 'testing.md'), markdown)
  }
)
