import camelCase from 'camelcase'
import { exec } from 'child_process'
import fs from 'fs/promises'
import { join } from 'path'
import semver from 'semver'
import yaml from 'yaml'

const specPath = join(__dirname, '..', 'specs.yml')
function generateOperationId(method, path) {
  // Split the path into segments and remove any empty segments or path parameters
  const segments = path
    .split('/')
    .filter(
      (seg) =>
        'string' === typeof seg &&
        !['api'].includes(seg) &&
        semver.valid(semver.coerce(seg)) === null
    )
    .map((seg) => seg.trim())
    .filter((seg) => seg.length > 0)
  segments.unshift(method)
  const formatted = segments.map((seg) => {
    if (seg.startsWith('{') || seg.endsWith('}')) {
      seg = '{id}'
    }
    return camelCase(seg, { pascalCase: true, preserveConsecutiveUppercase: true })
  })
  const stringified = formatted
    .join('_')
    .replace(/s_\{id\}$/gm, '')
    .replace(/s_\{id\}/gm, '')
    .replace(/Get([a-zA-Z_]+)s$/gm, 'list$1')
    .replace(/Post([a-zA-Z_]+)s$/gm, 'create$1')
    .replace(/Post([a-zA-Z_]+)FormData$/gm, 'create$1')
    .replace(/Get([a-zA-Z_]+)s_New$/gm, 'request$1')
    .replace(/Patch([a-zA-Z_]+)/gm, 'update$1')
    .replace(/^Post_([a-zA-Z_]+)_([a-zA-Z_]+)$/gm, '$2_$1')
    .replace('_{id}', 'ById')
  // return stringified
  return camelCase(stringified, { pascalCase: false, preserveConsecutiveUppercase: true })
}

fs.readFile(specPath, 'utf8').then(async (spec) => {
  const parsed = yaml.parse(spec)
  /**
   * First - Fix the Paths to include an Operation ID
   */
  for (const path in parsed.paths) {
    for (const method in parsed.paths[path]) {
      const operationId = generateOperationId(method, path)
      parsed.paths[path][method].operationId = operationId
    }
  }
  // write the update yaml file back
  await fs.writeFile(specPath, yaml.stringify(parsed))
  // write the definition as a typescript file
  const contents = `import type { OpenAPIV3 } from 'openapi-types';
export const definitions: OpenAPIV3.Document = ${JSON.stringify(parsed, null, 2)}`
  await fs.writeFile(join(__dirname, '..', 'src', 'OpenApiDefinitions.ts'), contents)
  // run the type generator
  const buffer: Buffer = await new Promise((resolve, reject) => {
    const stdOutBuffer: any = []
    const { stdout } = exec(`npx openapi-client-axios-typegen ${specPath}`)
    if (stdout) {
      stdout.on('data', (data) => stdOutBuffer.push(Buffer.from(data)))
      stdout.on('end', () => resolve(Buffer.concat(stdOutBuffer)))
      stdout.on('error', (error) => reject(error))
    } else {
      reject(new Error('No stdout'))
    }
  })
  const clientTypesFile = join(__dirname, '..', 'src', 'ClientTypes.ts')
  await fs.writeFile(clientTypesFile, buffer)
})
