import type { AxiosRequestConfig, AxiosResponse } from 'axios'
import crypto from 'crypto'
import { DateTime } from 'luxon'
import { OpenAPIClientAxios } from 'openapi-client-axios'
import qs from 'qs'
import type {
  AxiosResponseWithMetaData,
  Client as DocFoxAxiosClient,
  InternalAxiosRequestConfigWithMetaData,
  OperationMethodResponses,
  OperationMethods,
} from './ClientTypes'
import { definitionDocument } from './Definitions'
import { DocFoxAuthenticationError, DocFoxError } from './Errors'
export * from './ClientTypes'

export type ExtractNestedDataType<R> = R extends { data: { data: infer T } } ? T : any

export type AxiosInterceptorCallback<V> = (value: V) => V | Promise<V>
export type AxiosRequestInterceptorCallback = AxiosInterceptorCallback<
  InternalAxiosRequestConfigWithMetaData<any>
>
export type AxiosResponseInterceptorCallback = AxiosInterceptorCallback<
  AxiosResponseWithMetaData<any>
>
export type AxiosInterceptorCallbackType = AxiosRequestInterceptorCallback &
  AxiosResponseInterceptorCallback

export interface AuthTokenInformation {
  token?: string | undefined
  expiresAt?: DateTime | undefined
}

export interface AuthTokenGeneratorFunction {
  (key: string, secret: string): Promise<AuthTokenInformation>
}

export class DoxFoxClientInterceptor<T = AxiosInterceptorCallbackType> {
  readonly #which: 'request' | 'response'
  readonly #getDocFoxAxiosClientInstance: () => DocFoxAxiosClient
  constructor(
    which: 'request' | 'response',
    getDocFoxAxiosClientInstance: () => DocFoxAxiosClient
  ) {
    this.#which = which
    this.#getDocFoxAxiosClientInstance = getDocFoxAxiosClientInstance
  }

  public use(
    interceptor: T | null | undefined,
    errorInterceptor: (error: Error) => Promise<Error>
  ) {
    const instance = this.#getDocFoxAxiosClientInstance()
    // @ts-ignore
    return instance.interceptors[this.#which].use(interceptor, errorInterceptor)
  }
}

export class DoxFoxClientInterceptors {
  readonly #getDocFoxAxiosClientInstance: () => DocFoxAxiosClient
  constructor(getDocFoxAxiosClientInstance: () => DocFoxAxiosClient) {
    this.#getDocFoxAxiosClientInstance = getDocFoxAxiosClientInstance
  }

  public get request() {
    return new DoxFoxClientInterceptor<AxiosRequestInterceptorCallback>(
      'request',
      this.#getDocFoxAxiosClientInstance
    )
  }

  public get response() {
    return new DoxFoxClientInterceptor<AxiosResponseInterceptorCallback>(
      'response',
      this.#getDocFoxAxiosClientInstance
    )
  }
}

export class DocFoxClient {
  readonly #client: DocFoxAxiosClient
  readonly #authentication: AuthTokenInformation = {}
  readonly #generator?: AuthTokenGeneratorFunction | undefined
  readonly #key: string
  readonly #secret: string
  #compatibilityMode = false

  constructor(
    key: string,
    secret: string,
    generator?: AuthTokenGeneratorFunction | undefined,
    compatibilityMode = false
  ) {
    this.setCompatibilityMode(compatibilityMode)
    const api = new OpenAPIClientAxios({
      definition: definitionDocument,
      quick: false,
      withServer: {
        url: 'https://www.docfoxapp.com',
        description: 'DocFox API',
      },
      axiosConfigDefaults: {
        baseURL: 'https://www.docfoxapp.com',
        validateStatus: () => true,
        headers: {
          'content-type': 'application/json',
          'accept': 'application/vnd.api+json',
        },
      },
    })
    api.withServer({
      url: 'https://www.docfoxapp.com',
      description: 'DocFox API',
    })
    this.#client = api.initSync<DocFoxAxiosClient>()
    this.#key = key
    this.#secret = secret
    this.#generator = generator
    this.#client.defaults.baseURL = 'https://www.docfoxapp.com'
    this.#client.interceptors.request.use(
      async (config: InternalAxiosRequestConfigWithMetaData) => {
        config.metadata = {}
        const now = DateTime.now()
        const excludedPaths = ['/authentications/new', '/tokens/new']
        const excluded = excludedPaths.some((path) => config.url?.endsWith(path))
        if (!excluded) {
          const expired =
            !this.#authentication.token ||
            !this.#authentication.expiresAt ||
            this.#authentication.expiresAt <= now
          if (expired) {
            await this.#updateAuthentication()
            config.metadata.reauthenticated = true
          }
          if (this.#authentication.token) {
            config.headers.Authorization = `Bearer ${this.#authentication.token}`
          } else {
            config.headers.Authorization = ''
          }
        }
        config.metadata.startTime = new Date()
        return config
      },
      (error) => Promise.reject(error)
    )
    this.#client.interceptors.response.use(
      (response: AxiosResponseWithMetaData) => {
        response.config.metadata.endTime = new Date()
        response.duration = response.config.metadata.endTime - response.config.metadata.startTime
        return response
      },
      (error) => {
        if (error.config) {
          error.config.metadata.endTime = new Date()
          error.duration = error.config.metadata.endTime - error.config.metadata.startTime
        } else {
          error.duration = 0
        }
        return Promise.reject(error)
      }
    )
  }

  public get baseUrl(): string {
    return this.#client.defaults.baseURL || ''
  }

  public get client() {
    return this.#client
  }

  public get definitions() {
    return this.#client.api.definition
  }

  public get interceptors() {
    return new DoxFoxClientInterceptors(() => this.#client)
  }

  public setAuth(auth: AuthTokenInformation)
  public setAuth(auth: string | undefined, expiresAt?: DateTime | Date | string | undefined)
  public setAuth(
    auth: AuthTokenInformation | string | undefined,
    expiresAt?: DateTime | Date | string | undefined
  ) {
    const authObject: AuthTokenInformation = {}
    if ('object' === typeof auth) {
      authObject.token = auth.token
      authObject.expiresAt = auth.expiresAt
    }
    if ('string' === typeof auth) {
      authObject.token = auth
    }
    if ('string' === typeof expiresAt) {
      authObject.expiresAt = DateTime.fromISO(expiresAt)
    } else if ('object' === typeof expiresAt && expiresAt instanceof Date) {
      authObject.expiresAt = DateTime.fromJSDate(expiresAt)
    } else if ('object' === typeof expiresAt && expiresAt instanceof DateTime) {
      authObject.expiresAt = expiresAt
    }
    if (authObject.expiresAt && !authObject.expiresAt.isValid) {
      authObject.expiresAt = undefined
    }
    this.#authentication.token = authObject.token
    this.#authentication.expiresAt = authObject.expiresAt
  }

  public clearAuth() {
    this.#authentication.token = undefined
    this.#authentication.expiresAt = undefined
  }

  /**
   * Enable or disable compatibility mode which allows URI's which
   * do not start with /api/v2 to be passed to the axios alias functions (get, post, etc)
   * @param value If compatibility mode should be enabled
   */
  public setCompatibilityMode(value: boolean) {
    // eslint-disable-next-line eqeqeq
    this.#compatibilityMode = true == value
  }

  async #updateAuthentication(): Promise<void> {
    let authentication: AuthTokenInformation | undefined
    try {
      if (this.#generator) {
        authentication = await this.#generator(this.#key, this.#secret)
      } else {
        authentication = await this.#defaultAuthenticationGenerator(this.#key, this.#secret)
      }
    } catch (error) {
      if (error instanceof DocFoxAuthenticationError) {
        authentication = undefined
      } else {
        throw error
      }
    }
    this.#authentication.token = authentication?.token || undefined
    this.#authentication.expiresAt = authentication?.expiresAt || undefined
  }

  async #defaultAuthenticationGenerator(
    key: string,
    secret: string
  ): Promise<AuthTokenInformation> {
    const nonceResponse = await this.#client.requestAuthentication(undefined, null, {
      headers: {
        'X-Client-Api-Key': key,
      },
    })
    if (
      nonceResponse.status !== 200 ||
      !(nonceResponse.data as OperationMethodResponses['requestAuthentication']['$200']).data
    ) {
      throw new DocFoxAuthenticationError('Failed to request authentication nonce', nonceResponse)
    }
    const hmac = crypto.createHmac('sha256', secret)
    hmac.update(
      (nonceResponse.data as OperationMethodResponses['requestAuthentication']['$200']).data!
        .attributes!.nonce
    )
    const signature = hmac.digest('hex')
    const jwtResponse = await this.#client.requestToken(undefined, null, {
      headers: {
        'X-Client-Api-Key': key,
        'X-Client-Signature': signature,
      },
    })
    if (
      jwtResponse.status !== 200 ||
      !(jwtResponse.data as OperationMethodResponses['requestToken']['$200']).data
    ) {
      throw new DocFoxAuthenticationError('Failed to request authentication token', jwtResponse)
    }
    const token = (jwtResponse.data as OperationMethodResponses['requestToken']['$200']).data!
      .attributes!.token
    const expiresAt = DateTime.fromISO(
      (jwtResponse.data as OperationMethodResponses['requestToken']['$200']).data!.attributes!
        .expires_at
    )
    return {
      token,
      expiresAt,
    }
  }

  async #doClientRequestWithAuthenticationRetry<T extends AxiosResponseWithMetaData<any>>(
    request: () => Promise<T>
  ): Promise<T> {
    try {
      const res = await request()
      if (res.status === 401) {
        await this.#updateAuthentication()
        return await request()
      }
      return res
    } catch (error) {
      if (error instanceof DocFoxAuthenticationError) {
        await this.#updateAuthentication()
        return await request()
      } else if (error.code && error.code === 'ERR_CANCELED') {
        return {
          status: 0,
          statusText: 'Canceled',
          headers: {},
          config: {},
          data: undefined,
          duration: 0,
        } as T
      } else {
        return {
          status: 0,
          statusText: error.message,
          headers: {},
          config: {},
          data: error,
          duration: 0,
        } as T
      }
    }
  }

  /**
   * A proxy for the Axios getUri function
   * @param config Axios Request Configuration
   * @returns The URI for the request
   * @see [Axios Documentation](https://axios-http.com/docs/api_intro)
   */
  public getUri(config?: AxiosRequestConfig): string {
    if (this.#compatibilityMode && config && config.url) {
      const url = new URL(config.url)
      if (!url.pathname.startsWith('/api/v2')) {
        url.pathname = `/api/v2${url.pathname}`
        config.url = url.toString()
      }
    }
    return this.#client.getUri(config)
  }

  /**
   * A proxy for the Axios request function
   * @param config Axios Request Configuration
   * @returns An Axios Response with MetaData
   * @see [Axios Documentation](https://axios-http.com/docs/api_intro)
   */
  public async request<
    T = any,
    R extends AxiosResponseWithMetaData<any, any> = AxiosResponseWithMetaData<T, any>,
    D = any,
  >(config: AxiosRequestConfig<D>): Promise<R> {
    if (this.#compatibilityMode && config && config.url) {
      const url = new URL(config.url)
      if (!url.pathname.startsWith('/api/v2')) {
        url.pathname = `/api/v2${url.pathname}`
        config.url = url.toString()
      }
    }
    return await this.#doClientRequestWithAuthenticationRetry<R>(
      this.#client.request.bind(this.#client, config)
    )
  }

  /**
   * A proxy for the Axios get function
   * @param url Pathname of the request
   * @param config Additional Axios Request Configuration Overrides
   * @returns An Axios Response with MetaData
   * @see [Axios Documentation](https://axios-http.com/docs/api_intro)
   */
  public async get<
    T = any,
    R extends AxiosResponseWithMetaData<any, any> = AxiosResponseWithMetaData<T, any>,
    D = any,
  >(url: string, config?: AxiosRequestConfig<D>): Promise<R> {
    if (this.#compatibilityMode) {
      const Url = new URL(url)
      if (!Url.pathname.startsWith('/api/v2')) {
        Url.pathname = `/api/v2${Url.pathname}`
        url = Url.toString()
      }
    }
    if (this.#compatibilityMode && config && config.url) {
      const Url = new URL(config.url)
      if (!Url.pathname.startsWith('/api/v2')) {
        Url.pathname = `/api/v2${Url.pathname}`
        config.url = Url.toString()
      }
    }
    return await this.#doClientRequestWithAuthenticationRetry<R>(
      this.#client.get.bind(this.#client, url, config)
    )
  }

  /**
   * Helper function to simplify the retrieval of all resources for a given path
   * @param url Pathname of the request
   * @param config Additional Axios Request Configuration Overrides
   * @returns An Axios Response with MetaData
   */
  public async getAll<
    T = any,
    R extends AxiosResponseWithMetaData<any, any> = AxiosResponseWithMetaData<T, any>,
    D = any,
  >(url: string, config?: AxiosRequestConfig<D>): Promise<Array<T>> {
    const toReturn: Map<string, T> = new Map()
    if (this.#compatibilityMode) {
      const Url = new URL(url)
      if (!Url.pathname.startsWith('/api/v2')) {
        Url.pathname = `/api/v2${Url.pathname}`
        url = Url.toString()
      }
    }
    if (this.#compatibilityMode && config && config.url) {
      const Url = new URL(config.url)
      if (!Url.pathname.startsWith('/api/v2')) {
        Url.pathname = `/api/v2${Url.pathname}`
        config.url = Url.toString()
      }
    }
    let uri = url
    const query = {} as any
    if (uri.includes('?')) {
      const [path, queryString] = uri.split('?')
      uri = path
      Object.assign(query, qs.parse(queryString))
    }
    let page = 1
    let total = 0
    while (page === 1 || (total > 0 && toReturn.size < total)) {
      const pageQuery = Object.assign({}, query, {
        page,
      })
      const pageUrl = `${uri}?${qs.stringify(pageQuery)}`
      const response = await this.#doClientRequestWithAuthenticationRetry<R>(
        this.#client.get.bind(this.#client, pageUrl, config)
      )
      if (200 === response.status) {
        const {
          data: { data, meta } = {
            data: [],
            meta: {
              total: 0,
              per_page: 10,
              page,
            },
          },
        } = response
        total = meta?.total || 0
        if (data && Array.isArray(data)) {
          data.forEach((item) => {
            toReturn.set(JSON.stringify(item), item)
          })
        }
      }
      page++
    }
    return Array.from(toReturn.values())
  }

  /**
   * A proxy for the Axios delete function
   * @param url Pathname of the request
   * @param config Additional Axios Request Configuration Overrides
   * @returns An Axios Response with MetaData
   * @see [Axios Documentation](https://axios-http.com/docs/api_intro)
   */
  public async delete<
    T = any,
    R extends AxiosResponseWithMetaData<any, any> = AxiosResponseWithMetaData<T, any>,
    D = any,
  >(url: string, config?: AxiosRequestConfig<D>): Promise<R> {
    if (this.#compatibilityMode) {
      const Url = new URL(url)
      if (!Url.pathname.startsWith('/api/v2')) {
        Url.pathname = `/api/v2${Url.pathname}`
        url = Url.toString()
      }
    }
    if (this.#compatibilityMode && config && config.url) {
      const Url = new URL(config.url)
      if (!Url.pathname.startsWith('/api/v2')) {
        Url.pathname = `/api/v2${Url.pathname}`
        config.url = Url.toString()
      }
    }
    return await this.#doClientRequestWithAuthenticationRetry<R>(
      this.#client.delete.bind(this.#client, url, config)
    )
  }

  /**
   * A proxy for the Axios head function
   * @param url Pathname of the request
   * @param config Additional Axios Request Configuration Overrides
   * @returns An Axios Response with MetaData
   * @see [Axios Documentation](https://axios-http.com/docs/api_intro)
   */
  public async head<
    T = any,
    R extends AxiosResponseWithMetaData<any, any> = AxiosResponseWithMetaData<T, any>,
    D = any,
  >(url: string, config?: AxiosRequestConfig<D>): Promise<R> {
    if (this.#compatibilityMode) {
      const Url = new URL(url)
      if (!Url.pathname.startsWith('/api/v2')) {
        Url.pathname = `/api/v2${Url.pathname}`
        url = Url.toString()
      }
    }
    if (this.#compatibilityMode && config && config.url) {
      const Url = new URL(config.url)
      if (!Url.pathname.startsWith('/api/v2')) {
        Url.pathname = `/api/v2${Url.pathname}`
        config.url = Url.toString()
      }
    }
    return await this.#doClientRequestWithAuthenticationRetry<R>(
      this.#client.head.bind(this.#client, url, config)
    )
  }

  /**
   * A proxy for the Axios options function
   * @param url Pathname of the request
   * @param config Additional Axios Request Configuration Overrides
   * @returns An Axios Response with MetaData
   * @see [Axios Documentation](https://axios-http.com/docs/api_intro)
   */
  public async options<
    T = any,
    R extends AxiosResponseWithMetaData<any, any> = AxiosResponseWithMetaData<T, any>,
    D = any,
  >(url: string, config?: AxiosRequestConfig<D>): Promise<R> {
    if (this.#compatibilityMode) {
      const Url = new URL(url)
      if (!Url.pathname.startsWith('/api/v2')) {
        Url.pathname = `/api/v2${Url.pathname}`
        url = Url.toString()
      }
    }
    if (this.#compatibilityMode && config && config.url) {
      const Url = new URL(config.url)
      if (!Url.pathname.startsWith('/api/v2')) {
        Url.pathname = `/api/v2${Url.pathname}`
        config.url = Url.toString()
      }
    }
    return await this.#doClientRequestWithAuthenticationRetry<R>(
      this.#client.options.bind(this.#client, url, config)
    )
  }

  /**
   * A proxy for the Axios post function
   * @param url Pathname of the request
   * @param data Data to send with the request
   * @param config Additional Axios Request Configuration Overrides
   * @returns An Axios Response with MetaData
   * @see [Axios Documentation](https://axios-http.com/docs/api_intro)
   */
  public async post<
    T = any,
    R extends AxiosResponseWithMetaData<any, any> = AxiosResponseWithMetaData<T, any>,
    D = any,
  >(url: string, data?: D, config?: AxiosRequestConfig<D>): Promise<R> {
    if (this.#compatibilityMode) {
      const Url = new URL(url)
      if (!Url.pathname.startsWith('/api/v2')) {
        Url.pathname = `/api/v2${Url.pathname}`
        url = Url.toString()
      }
    }
    if (this.#compatibilityMode && config && config.url) {
      const Url = new URL(config.url)
      if (!Url.pathname.startsWith('/api/v2')) {
        Url.pathname = `/api/v2${Url.pathname}`
        config.url = Url.toString()
      }
    }
    return await this.#doClientRequestWithAuthenticationRetry<R>(
      this.#client.post.bind(this.#client, url, data, config)
    )
  }

  /**
   * A proxy for the Axios put function
   * @param url Pathname of the request
   * @param data Data to send with the request
   * @param config Additional Axios Request Configuration Overrides
   * @returns An Axios Response with MetaData
   * @see [Axios Documentation](https://axios-http.com/docs/api_intro)
   */
  public async put<
    T = any,
    R extends AxiosResponseWithMetaData<any, any> = AxiosResponseWithMetaData<T, any>,
    D = any,
  >(url: string, data?: D, config?: AxiosRequestConfig<D>): Promise<R> {
    if (this.#compatibilityMode) {
      const Url = new URL(url)
      if (!Url.pathname.startsWith('/api/v2')) {
        Url.pathname = `/api/v2${Url.pathname}`
        url = Url.toString()
      }
    }
    if (this.#compatibilityMode && config && config.url) {
      const Url = new URL(config.url)
      if (!Url.pathname.startsWith('/api/v2')) {
        Url.pathname = `/api/v2${Url.pathname}`
        config.url = Url.toString()
      }
    }
    return await this.#doClientRequestWithAuthenticationRetry<R>(
      this.#client.put.bind(this.#client, url, data, config)
    )
  }

  /**
   * A proxy for the Axios patch function
   * @param url Pathname of the request
   * @param data Data to send with the request
   * @param config Additional Axios Request Configuration Overrides
   * @returns An Axios Response with MetaData
   * @see [Axios Documentation](https://axios-http.com/docs/api_intro)
   */
  public async patch<
    T = any,
    R extends AxiosResponseWithMetaData<any, any> = AxiosResponseWithMetaData<T, any>,
    D = any,
  >(url: string, data?: D, config?: AxiosRequestConfig<D>): Promise<R> {
    if (this.#compatibilityMode) {
      const Url = new URL(url)
      if (!Url.pathname.startsWith('/api/v2')) {
        Url.pathname = `/api/v2${Url.pathname}`
        url = Url.toString()
      }
    }
    if (this.#compatibilityMode && config && config.url) {
      const Url = new URL(config.url)
      if (!Url.pathname.startsWith('/api/v2')) {
        Url.pathname = `/api/v2${Url.pathname}`
        config.url = Url.toString()
      }
    }
    return await this.#doClientRequestWithAuthenticationRetry<R>(
      this.#client.patch.bind(this.#client, url, data, config)
    )
  }

  /**
   * A proxy for the Axios postForm function
   * @param url Pathname of the request
   * @param data Data to send with the request
   * @param config Additional Axios Request Configuration Overrides
   * @returns An Axios Response with MetaData
   * @see [Axios Documentation](https://axios-http.com/docs/api_intro)
   */
  public async postForm<
    T = any,
    R extends AxiosResponseWithMetaData<any, any> = AxiosResponseWithMetaData<T, any>,
    D = any,
  >(url: string, data?: D, config?: AxiosRequestConfig<D>): Promise<R> {
    if (this.#compatibilityMode) {
      const Url = new URL(url)
      if (!Url.pathname.startsWith('/api/v2')) {
        Url.pathname = `/api/v2${Url.pathname}`
        url = Url.toString()
      }
    }
    if (this.#compatibilityMode && config && config.url) {
      const Url = new URL(config.url)
      if (!Url.pathname.startsWith('/api/v2')) {
        Url.pathname = `/api/v2${Url.pathname}`
        config.url = Url.toString()
      }
    }
    return await this.#doClientRequestWithAuthenticationRetry<R>(
      this.#client.postForm.bind(this.#client, url, data, config)
    )
  }

  /**
   * A proxy for the Axios putForm function
   * @param url Pathname of the request
   * @param data Data to send with the request
   * @param config Additional Axios Request Configuration Overrides
   * @returns An Axios Response with MetaData
   * @see [Axios Documentation](https://axios-http.com/docs/api_intro)
   */
  public async putForm<
    T = any,
    R extends AxiosResponseWithMetaData<any, any> = AxiosResponseWithMetaData<T, any>,
    D = any,
  >(url: string, data?: D, config?: AxiosRequestConfig<D>): Promise<R> {
    if (this.#compatibilityMode) {
      const Url = new URL(url)
      if (!Url.pathname.startsWith('/api/v2')) {
        Url.pathname = `/api/v2${Url.pathname}`
        url = Url.toString()
      }
    }
    if (this.#compatibilityMode && config && config.url) {
      const Url = new URL(config.url)
      if (!Url.pathname.startsWith('/api/v2')) {
        Url.pathname = `/api/v2${Url.pathname}`
        config.url = Url.toString()
      }
    }
    return await this.#doClientRequestWithAuthenticationRetry<R>(
      this.#client.putForm.bind(this.#client, url, data, config)
    )
  }

  /**
   * A proxy for the Axios patchForm function
   * @param url Pathname of the request
   * @param data Data to send with the request
   * @param config Additional Axios Request Configuration Overrides
   * @returns An Axios Response with MetaData
   * @see [Axios Documentation](https://axios-http.com/docs/api_intro)
   */
  public async patchForm<
    T = any,
    R extends AxiosResponseWithMetaData<any, any> = AxiosResponseWithMetaData<T, any>,
    D = any,
  >(url: string, data?: D, config?: AxiosRequestConfig<D>): Promise<R> {
    if (this.#compatibilityMode) {
      const Url = new URL(url)
      if (!Url.pathname.startsWith('/api/v2')) {
        Url.pathname = `/api/v2${Url.pathname}`
        url = Url.toString()
      }
    }
    if (this.#compatibilityMode && config && config.url) {
      const Url = new URL(config.url)
      if (!Url.pathname.startsWith('/api/v2')) {
        Url.pathname = `/api/v2${Url.pathname}`
        config.url = Url.toString()
      }
    }
    return await this.#doClientRequestWithAuthenticationRetry<R>(
      this.#client.patchForm.bind(this.#client, url, data, config)
    )
  }

  /**
   * Helper function to simplify the retrieval of all resources for a given operation
   * @param operation Operation Method to call
   * @param parameters Parameters to pass to the operation excluding page and per_page
   * @param payload Payload to pass to the operation
   * @param config Additional Axios Request Configuration Overrides
   * @returns An array of the extracted data from the response
   */
  public async getAllFromOperationMethod<
    O extends keyof OperationMethods,
    P = Omit<Parameters<OperationMethods[O]>[0], 'page' | 'per_page'>,
    D = Parameters<OperationMethods[O]>[1],
    C = Parameters<OperationMethods[O]>[2],
    R extends AxiosResponseWithMetaData<any, any> = Awaited<ReturnType<OperationMethods[O]>>,
    T = ExtractNestedDataType<R>,
  >(operation: O, parameters?: P, payload?: D, config?: C): Promise<T> {
    const toReturn: Map<string, T> = new Map()
    let page = 1
    let total = 0
    if ('object' !== typeof parameters || null === parameters) {
      parameters = {} as P
    }
    while (page === 1 || (total > 0 && toReturn.size < total)) {
      const response = await this.#doClientRequestWithAuthenticationRetry<R>(
        this.#client[operation].bind(
          this.#client,
          { ...parameters, page, per_page: 10 },
          payload,
          config
        )
      )
      if (200 === response.status) {
        const {
          data: { data, meta } = {
            data: [],
            meta: {
              total: 0,
              per_page: 10,
              page,
            },
          },
        } = response
        total = meta?.total || 0
        if (data && Array.isArray(data)) {
          data.forEach((item) => {
            toReturn.set(JSON.stringify(item), item)
          })
        }
      }
      page++
    }
    return Array.from(toReturn.values()) as T
  }

  /**
   * Operation Method for `GET` `/api/v2/authentications/new`
   * @tag Authentication
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Authentication/paths/~1api~1v2~1authentications~1new/get)
   */
  public async requestAuthentication(
    params: { 'X-Client-Api-Key': string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['requestAuthentication']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['requestAuthentication']>>
    >(this.#client.requestAuthentication.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/tokens/new`
   * @tag Authentication
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Authentication/paths/~1api~1v2~1tokens~1new/get)
   */
  public async requestToken(
    params: { 'X-Client-Signature': string; 'X-Client-Api-Key': string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['requestToken']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['requestToken']>>
    >(this.#client.requestToken.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/kyc_applications`
   * @tag KYC Applications
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Applications/paths/~1api~1v2~1kyc_applications/get)
   */
  public async listKycApplication(
    params: { filter?: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listKycApplication']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listKycApplication']>>
    >(this.#client.listKycApplication.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `POST` `/api/v2/kyc_applications`
   * @tag KYC Applications
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Applications/paths/~1api~1v2~1kyc_applications/post)
   */
  public async createKycApplication(
    params: {},
    payload?: { data: Record<string, any> },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['createKycApplication']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['createKycApplication']>>
    >(this.#client.createKycApplication.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/kyc_applications/{kyc_application_id}`
   * @tag KYC Applications
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Applications/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D/get)
   */
  public async getKycApplication(
    params: { kyc_application_id: string; include?: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getKycApplication']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getKycApplication']>>
    >(this.#client.getKycApplication.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `DELETE` `/api/v2/kyc_applications/{kyc_application_id}`
   * @tag KYC Applications
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Applications/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D/delete)
   */
  public async deleteKycApplication(
    params: { kyc_application_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['deleteKycApplication']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['deleteKycApplication']>>
    >(this.#client.deleteKycApplication.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `PATCH` `/api/v2/kyc_applications/{kyc_application_id}/approve`
   * @tag KYC Applications
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Applications/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D~1approve/patch)
   */
  public async updateKycApplicationApprove(
    params: { kyc_application_id: string },
    payload?: { data: Record<string, any> },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['updateKycApplicationApprove']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['updateKycApplicationApprove']>>
    >(this.#client.updateKycApplicationApprove.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `PATCH` `/api/v2/kyc_applications/{kyc_application_id}/unapprove`
   * @tag KYC Applications
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Applications/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D~1unapprove/patch)
   */
  public async updateKycApplicationUnapprove(
    params: { kyc_application_id: string },
    payload?: { data: Record<string, any> },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['updateKycApplicationUnapprove']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['updateKycApplicationUnapprove']>>
    >(this.#client.updateKycApplicationUnapprove.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `PATCH` `/api/v2/kyc_applications/{kyc_application_id}/transfer`
   * @tag KYC Applications
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Applications/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D~1transfer/patch)
   */
  public async updateKycApplicationTransfer(
    params: { kyc_application_id: string },
    payload?: { data: Record<string, any> },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['updateKycApplicationTransfer']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['updateKycApplicationTransfer']>>
    >(this.#client.updateKycApplicationTransfer.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `POST` `/api/v2/kyc_applications/{kyc_application_id}/archive`
   * @tag KYC Applications
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Applications/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D~1archive/post)
   */
  public async archiveKycApplication(
    params: { kyc_application_id: string },
    payload?: { data: Record<string, any> },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['archiveKycApplication']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['archiveKycApplication']>>
    >(this.#client.archiveKycApplication.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `POST` `/api/v2/kyc_applications/{kyc_application_id}/unarchive`
   * @tag KYC Applications
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Applications/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D~1unarchive/post)
   */
  public async unarchiveKycApplication(
    params: { kyc_application_id: string },
    payload?: { data: Record<string, any> },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['unarchiveKycApplication']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['unarchiveKycApplication']>>
    >(this.#client.unarchiveKycApplication.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/kyc_applications/{kyc_application_id}/status_summaries`
   * @tag Status Summaries
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Status-Summaries/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D~1status_summaries/get)
   */
  public async listKycApplicationStatusSummarie(
    params: { kyc_application_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listKycApplicationStatusSummarie']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listKycApplicationStatusSummarie']>>
    >(this.#client.listKycApplicationStatusSummarie.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/kyc_applications/{kyc_application_id}/profiles`
   * @tag Profiles
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Profiles/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D~1profiles/get)
   */
  public async listKycApplicationProfile(
    params: { kyc_application_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listKycApplicationProfile']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listKycApplicationProfile']>>
    >(this.#client.listKycApplicationProfile.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/profiles/{profile_id}`
   * @tag Profiles
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Profiles/paths/~1api~1v2~1profiles~1%7Bprofile_id%7D/get)
   */
  public async getProfile(
    params: { profile_id: string; include?: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getProfile']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getProfile']>>
    >(this.#client.getProfile.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/profiles/{profile_id}/names`
   * @tag KYC Profile Names
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Names/paths/~1api~1v2~1profiles~1%7Bprofile_id%7D~1names/get)
   */
  public async listProfileName(
    params: { profile_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listProfileName']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listProfileName']>>
    >(this.#client.listProfileName.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `POST` `/api/v2/profiles/{profile_id}/names`
   * @tag KYC Profile Names
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Names/paths/~1api~1v2~1profiles~1%7Bprofile_id%7D~1names/post)
   */
  public async createProfileName(
    params: { name_id: string; profile_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['createProfileName']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['createProfileName']>>
    >(this.#client.createProfileName.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/names/{name_id}`
   * @tag KYC Profile Names
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Names/paths/~1api~1v2~1names~1%7Bname_id%7D/get)
   */
  public async getName(
    params: { name_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getName']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getName']>>
    >(this.#client.getName.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `DELETE` `/api/v2/names/{name_id}`
   * @tag KYC Profile Names
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Names/paths/~1api~1v2~1names~1%7Bname_id%7D/delete)
   */
  public async deleteName(
    params: { name_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['deleteName']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['deleteName']>>
    >(this.#client.deleteName.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `PATCH` `/api/v2/names/{name_id}`
   * @tag KYC Profile Names
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Names/paths/~1api~1v2~1names~1%7Bname_id%7D/patch)
   */
  public async updateName(
    params: { name_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['updateName']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['updateName']>>
    >(this.#client.updateName.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/profiles/{profile_id}/numbers`
   * @tag KYC Profile Numbers
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Numbers/paths/~1api~1v2~1profiles~1%7Bprofile_id%7D~1numbers/get)
   */
  public async listProfileNumber(
    params: { profile_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listProfileNumber']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listProfileNumber']>>
    >(this.#client.listProfileNumber.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `POST` `/api/v2/profiles/{profile_id}/numbers`
   * @tag KYC Profile Numbers
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Numbers/paths/~1api~1v2~1profiles~1%7Bprofile_id%7D~1numbers/post)
   */
  public async createProfileNumber(
    params: { number_id: string; profile_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['createProfileNumber']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['createProfileNumber']>>
    >(this.#client.createProfileNumber.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/numbers/{number_id}`
   * @tag KYC Profile Numbers
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Numbers/paths/~1api~1v2~1numbers~1%7Bnumber_id%7D/get)
   */
  public async getNumber(
    params: { number_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getNumber']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getNumber']>>
    >(this.#client.getNumber.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `DELETE` `/api/v2/numbers/{number_id}`
   * @tag KYC Profile Numbers
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Numbers/paths/~1api~1v2~1numbers~1%7Bnumber_id%7D/delete)
   */
  public async deleteNumber(
    params: { number_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['deleteNumber']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['deleteNumber']>>
    >(this.#client.deleteNumber.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `PATCH` `/api/v2/numbers/{number_id}`
   * @tag KYC Profile Numbers
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Numbers/paths/~1api~1v2~1numbers~1%7Bnumber_id%7D/patch)
   */
  public async updateNumber(
    params: { number_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['updateNumber']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['updateNumber']>>
    >(this.#client.updateNumber.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/profiles/{profile_id}/contacts`
   * @tag KYC Profile Contact Informations
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Contact-Informations/paths/~1api~1v2~1profiles~1%7Bprofile_id%7D~1contacts/get)
   */
  public async listProfileContact(
    params: { profile_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listProfileContact']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listProfileContact']>>
    >(this.#client.listProfileContact.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `POST` `/api/v2/profiles/{profile_id}/contacts`
   * @tag KYC Profile Contact Informations
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Contact-Informations/paths/~1api~1v2~1profiles~1%7Bprofile_id%7D~1contacts/post)
   */
  public async createProfileContact(
    params: { contact_id: string; profile_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['createProfileContact']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['createProfileContact']>>
    >(this.#client.createProfileContact.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/contacts/{contact_id}`
   * @tag KYC Profile Contact Informations
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Contact-Informations/paths/~1api~1v2~1contacts~1%7Bcontact_id%7D/get)
   */
  public async getContact(
    params: { contact_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getContact']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getContact']>>
    >(this.#client.getContact.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `DELETE` `/api/v2/contacts/{contact_id}`
   * @tag KYC Profile Contact Informations
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Contact-Informations/paths/~1api~1v2~1contacts~1%7Bcontact_id%7D/delete)
   */
  public async deleteContact(
    params: { contact_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['deleteContact']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['deleteContact']>>
    >(this.#client.deleteContact.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `PATCH` `/api/v2/contacts/{contact_id}`
   * @tag KYC Profile Contact Informations
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Contact-Informations/paths/~1api~1v2~1contacts~1%7Bcontact_id%7D/patch)
   */
  public async updateContact(
    params: { contact_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['updateContact']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['updateContact']>>
    >(this.#client.updateContact.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/profiles/{profile_id}/addresses`
   * @tag KYC Profile Addresses
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Addresses/paths/~1api~1v2~1profiles~1%7Bprofile_id%7D~1addresses/get)
   */
  public async listProfileAddresse(
    params: { profile_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listProfileAddresse']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listProfileAddresse']>>
    >(this.#client.listProfileAddresse.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `POST` `/api/v2/profiles/{profile_id}/addresses`
   * @tag KYC Profile Addresses
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Addresses/paths/~1api~1v2~1profiles~1%7Bprofile_id%7D~1addresses/post)
   */
  public async createProfileAddresse(
    params: { address_id: string; profile_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['createProfileAddresse']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['createProfileAddresse']>>
    >(this.#client.createProfileAddresse.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/addresses/{address_id}`
   * @tag KYC Profile Addresses
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Addresses/paths/~1api~1v2~1addresses~1%7Baddress_id%7D/get)
   */
  public async getAddresse(
    params: { address_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getAddresse']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getAddresse']>>
    >(this.#client.getAddresse.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `DELETE` `/api/v2/addresses/{address_id}`
   * @tag KYC Profile Addresses
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Addresses/paths/~1api~1v2~1addresses~1%7Baddress_id%7D/delete)
   */
  public async deleteAddresse(
    params: { address_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['deleteAddresse']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['deleteAddresse']>>
    >(this.#client.deleteAddresse.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `PATCH` `/api/v2/addresses/{address_id}`
   * @tag KYC Profile Addresses
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Addresses/paths/~1api~1v2~1addresses~1%7Baddress_id%7D/patch)
   */
  public async updateAddresse(
    params: { address_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['updateAddresse']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['updateAddresse']>>
    >(this.#client.updateAddresse.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/profiles/{profile_id}/additional_details`
   * @tag KYC Profile Additional Details
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Additional-Details/paths/~1api~1v2~1profiles~1%7Bprofile_id%7D~1additional_details/get)
   */
  public async listProfileAdditionalDetail(
    params: { profile_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listProfileAdditionalDetail']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listProfileAdditionalDetail']>>
    >(this.#client.listProfileAdditionalDetail.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `POST` `/api/v2/profiles/{profile_id}/additional_details`
   * @tag KYC Profile Additional Details
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Additional-Details/paths/~1api~1v2~1profiles~1%7Bprofile_id%7D~1additional_details/post)
   */
  public async createProfileAdditionalDetail(
    params: { additional_detail_id: string; profile_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['createProfileAdditionalDetail']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['createProfileAdditionalDetail']>>
    >(this.#client.createProfileAdditionalDetail.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/additional_details/{additional_detail_id}`
   * @tag KYC Profile Additional Details
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Additional-Details/paths/~1api~1v2~1additional_details~1%7Badditional_detail_id%7D/get)
   */
  public async getAdditionalDetail(
    params: { additional_detail_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getAdditionalDetail']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getAdditionalDetail']>>
    >(this.#client.getAdditionalDetail.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `DELETE` `/api/v2/additional_details/{additional_detail_id}`
   * @tag KYC Profile Additional Details
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Additional-Details/paths/~1api~1v2~1additional_details~1%7Badditional_detail_id%7D/delete)
   */
  public async deleteAdditionalDetail(
    params: { additional_detail_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['deleteAdditionalDetail']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['deleteAdditionalDetail']>>
    >(this.#client.deleteAdditionalDetail.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `PATCH` `/api/v2/additional_details/{additional_detail_id}`
   * @tag KYC Profile Additional Details
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Profile-Additional-Details/paths/~1api~1v2~1additional_details~1%7Badditional_detail_id%7D/patch)
   */
  public async updateAdditionalDetail(
    params: { additional_detail_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['updateAdditionalDetail']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['updateAdditionalDetail']>>
    >(this.#client.updateAdditionalDetail.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/contacts/{contact_information_id}/invitation_tokens`
   * @tag Invitation Tokens
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Invitation-Tokens/paths/~1api~1v2~1contacts~1%7Bcontact_information_id%7D~1invitation_tokens/get)
   */
  public async listContactInvitationToken(
    params: { contact_information_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listContactInvitationToken']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listContactInvitationToken']>>
    >(this.#client.listContactInvitationToken.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `POST` `/api/v2/contacts/{contact_information_id}/invitation_tokens`
   * @tag Invitation Tokens
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Invitation-Tokens/paths/~1api~1v2~1contacts~1%7Bcontact_information_id%7D~1invitation_tokens/post)
   */
  public async createContactInvitationToken(
    params: { contact_information_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['createContactInvitationToken']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['createContactInvitationToken']>>
    >(this.#client.createContactInvitationToken.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/invitation_tokens/{invitation_token_id}`
   * @tag Invitation Tokens
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Invitation-Tokens/paths/~1api~1v2~1invitation_tokens~1%7Binvitation_token_id%7D/get)
   */
  public async getInvitationToken(
    params: { invitation_token_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getInvitationToken']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getInvitationToken']>>
    >(this.#client.getInvitationToken.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `PATCH` `/api/v2/invitation_tokens/{invitation_token_id}`
   * @tag Invitation Tokens
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Invitation-Tokens/paths/~1api~1v2~1invitation_tokens~1%7Binvitation_token_id%7D/patch)
   */
  public async updateInvitationToken(
    params: { contact_information_id: string; invitation_token_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['updateInvitationToken']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['updateInvitationToken']>>
    >(this.#client.updateInvitationToken.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/contacts/{contact_information_id}/data_requirement_permissions`
   * @tag Data Requirement Permissions
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Data-Requirement-Permissions/paths/~1api~1v2~1contacts~1%7Bcontact_information_id%7D~1data_requirement_permissions/get)
   */
  public async listContactDataRequirementPermission(
    params: { contact_information_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listContactDataRequirementPermission']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listContactDataRequirementPermission']>>
    >(
      this.#client.listContactDataRequirementPermission.bind(
        this.#client,
        params,
        undefined,
        config
      )
    )
  }

  /**
   * Operation Method for `GET` `/api/v2/data_requirement_permissions/{data_requirement_permission_id}`
   * @tag Data Requirement Permissions
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Data-Requirement-Permissions/paths/~1api~1v2~1data_requirement_permissions~1%7Bdata_requirement_permission_id%7D/get)
   */
  public async getDataRequirementPermission(
    params: { data_requirement_permission_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getDataRequirementPermission']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getDataRequirementPermission']>>
    >(this.#client.getDataRequirementPermission.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `DELETE` `/api/v2/data_requirement_permissions/{data_requirement_permission_id}`
   * @tag Data Requirement Permissions
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Data-Requirement-Permissions/paths/~1api~1v2~1data_requirement_permissions~1%7Bdata_requirement_permission_id%7D/delete)
   */
  public async deleteDataRequirementPermission(
    params: { data_requirement_permission_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['deleteDataRequirementPermission']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['deleteDataRequirementPermission']>>
    >(this.#client.deleteDataRequirementPermission.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `POST` `/api/v2/data_requirement_permissions`
   * @tag Data Requirement Permissions
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Data-Requirement-Permissions/paths/~1api~1v2~1data_requirement_permissions/post)
   */
  public async createDataRequirementPermission(
    params: {},
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['createDataRequirementPermission']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['createDataRequirementPermission']>>
    >(this.#client.createDataRequirementPermission.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/data_requirements/{data_requirement_id}/data_requirement_permissions`
   * @tag Data Requirement Permissions
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Data-Requirement-Permissions/paths/~1api~1v2~1data_requirements~1%7Bdata_requirement_id%7D~1data_requirement_permissions/get)
   */
  public async listDataRequirementDataRequirementPermission(
    params: { data_requirement_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listDataRequirementDataRequirementPermission']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listDataRequirementDataRequirementPermission']>>
    >(
      this.#client.listDataRequirementDataRequirementPermission.bind(
        this.#client,
        params,
        undefined,
        config
      )
    )
  }

  /**
   * Operation Method for `POST` `/api/v2/data_requirements/{data_requirement_id}/data_requirement_permissions`
   * @tag Data Requirement Permissions
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Data-Requirement-Permissions/paths/~1api~1v2~1data_requirements~1%7Bdata_requirement_id%7D~1data_requirement_permissions/post)
   */
  public async createDataRequirementDataRequirementPermission(
    params: { data_requirement_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['createDataRequirementDataRequirementPermission']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['createDataRequirementDataRequirementPermission']>>
    >(
      this.#client.createDataRequirementDataRequirementPermission.bind(
        this.#client,
        params,
        payload,
        config
      )
    )
  }

  /**
   * Operation Method for `POST` `/api/v2/contacts/{contact_id}/data_requirement_permissions`
   * @tag Data Requirement Permissions
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Data-Requirement-Permissions/paths/~1api~1v2~1contacts~1%7Bcontact_id%7D~1data_requirement_permissions/post)
   */
  public async createContactDataRequirementPermission(
    params: { contact_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['createContactDataRequirementPermission']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['createContactDataRequirementPermission']>>
    >(
      this.#client.createContactDataRequirementPermission.bind(
        this.#client,
        params,
        payload,
        config
      )
    )
  }

  /**
   * Operation Method for `GET` `/api/v2/kyc_applications/{kyc_application_id}/data_requirements`
   * @tag Data Requirements
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Data-Requirements/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D~1data_requirements/get)
   */
  public async listKycApplicationDataRequirement(
    params: { kyc_application_id: string; forms?: boolean; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listKycApplicationDataRequirement']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listKycApplicationDataRequirement']>>
    >(this.#client.listKycApplicationDataRequirement.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/data_requirements/{data_requirement_id}`
   * @tag Data Requirements
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Data-Requirements/paths/~1api~1v2~1data_requirements~1%7Bdata_requirement_id%7D/get)
   */
  public async getDataRequirement(
    params: { data_requirement_id: string; include?: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getDataRequirement']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getDataRequirement']>>
    >(this.#client.getDataRequirement.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/kyc_entity_templates`
   * @tag KYC Entity Templates
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Entity-Templates/paths/~1api~1v2~1kyc_entity_templates/get)
   */
  public async listKycEntityTemplate(
    params: { page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listKycEntityTemplate']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listKycEntityTemplate']>>
    >(this.#client.listKycEntityTemplate.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/kyc_entity_templates/{kyc_entity_template_id}`
   * @tag KYC Entity Templates
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KYC-Entity-Templates/paths/~1api~1v2~1kyc_entity_templates~1%7Bkyc_entity_template_id%7D/get)
   */
  public async getKycEntityTemplate(
    params: { kyc_entity_template_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getKycEntityTemplate']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getKycEntityTemplate']>>
    >(this.#client.getKycEntityTemplate.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/kyc_applications/{kyc_application_id}/data_comparisons`
   * @tag Data Comparisons
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Data-Comparisons/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D~1data_comparisons/get)
   */
  public async listKycApplicationDataComparison(
    params: { kyc_application_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listKycApplicationDataComparison']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listKycApplicationDataComparison']>>
    >(this.#client.listKycApplicationDataComparison.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/data_comparisons/{data_comparison_id}`
   * @tag Data Comparisons
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Data-Comparisons/paths/~1api~1v2~1data_comparisons~1%7Bdata_comparison_id%7D/get)
   */
  public async getDataComparison(
    params: { data_comparison_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getDataComparison']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getDataComparison']>>
    >(this.#client.getDataComparison.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/profiles/{profile_id}/data_comparisons`
   * @tag Data Comparisons
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Data-Comparisons/paths/~1api~1v2~1profiles~1%7Bprofile_id%7D~1data_comparisons/get)
   */
  public async listProfileDataComparison(
    params: { profile_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listProfileDataComparison']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listProfileDataComparison']>>
    >(this.#client.listProfileDataComparison.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/data_requirements/{data_requirement_id}/data_comparisons`
   * @tag Data Comparisons
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Data-Comparisons/paths/~1api~1v2~1data_requirements~1%7Bdata_requirement_id%7D~1data_comparisons/get)
   */
  public async listDataRequirementDataComparison(
    params: { data_requirement_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listDataRequirementDataComparison']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listDataRequirementDataComparison']>>
    >(this.#client.listDataRequirementDataComparison.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/data_comparisons/{data_comparison_id}/comparison_results`
   * @tag Comparison Results
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Comparison-Results/paths/~1api~1v2~1data_comparisons~1%7Bdata_comparison_id%7D~1comparison_results/get)
   */
  public async listDataComparisonComparisonResult(
    params: { data_comparison_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listDataComparisonComparisonResult']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listDataComparisonComparisonResult']>>
    >(this.#client.listDataComparisonComparisonResult.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/comparison_results/{comparison_result_id}`
   * @tag Comparison Results
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Comparison-Results/paths/~1api~1v2~1comparison_results~1%7Bcomparison_result_id%7D/get)
   */
  public async getComparisonResult(
    params: { comparison_result_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getComparisonResult']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getComparisonResult']>>
    >(this.#client.getComparisonResult.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/data_requirements/{data_requirement_id}/evidence_submissions`
   * @tag Evidence Submissions
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Evidence-Submissions/paths/~1api~1v2~1data_requirements~1%7Bdata_requirement_id%7D~1evidence_submissions/get)
   */
  public async listDataRequirementEvidenceSubmission(
    params: { data_requirement_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listDataRequirementEvidenceSubmission']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listDataRequirementEvidenceSubmission']>>
    >(
      this.#client.listDataRequirementEvidenceSubmission.bind(
        this.#client,
        params,
        undefined,
        config
      )
    )
  }

  /**
   * Operation Method for `GET` `/api/v2/evidence_submissions/{evidence_submission_id}`
   * @tag Evidence Submissions
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Evidence-Submissions/paths/~1api~1v2~1evidence_submissions~1%7Bevidence_submission_id%7D/get)
   */
  public async getEvidenceSubmission(
    params: { evidence_submission_id: string; include?: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getEvidenceSubmission']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getEvidenceSubmission']>>
    >(this.#client.getEvidenceSubmission.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `PATCH` `/api/v2/evidence_submissions/{evidence_submission_id}`
   * @tag Evidence Submissions
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Evidence-Submissions/paths/~1api~1v2~1evidence_submissions~1%7Bevidence_submission_id%7D/patch)
   */
  public async updateEvidenceSubmission(
    params: { evidence_submission_id: string; include?: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['updateEvidenceSubmission']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['updateEvidenceSubmission']>>
    >(this.#client.updateEvidenceSubmission.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `PATCH` `/api/v2/evidence_submissions/{evidence_submission_id}/reject`
   * @tag Evidence Submissions
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Evidence-Submissions/paths/~1api~1v2~1evidence_submissions~1%7Bevidence_submission_id%7D~1reject/patch)
   */
  public async updateEvidenceSubmissionReject(
    params: { evidence_submission_id: string; include?: string },
    payload?: { data: Record<string, any> },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['updateEvidenceSubmissionReject']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['updateEvidenceSubmissionReject']>>
    >(this.#client.updateEvidenceSubmissionReject.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `PATCH` `/api/v2/evidence_submissions/{evidence_submission_id}/approve`
   * @tag Evidence Submissions
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Evidence-Submissions/paths/~1api~1v2~1evidence_submissions~1%7Bevidence_submission_id%7D~1approve/patch)
   */
  public async updateEvidenceSubmissionApprove(
    params: { evidence_submission_id: string; include?: string },
    payload?: { data: Record<string, any> },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['updateEvidenceSubmissionApprove']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['updateEvidenceSubmissionApprove']>>
    >(this.#client.updateEvidenceSubmissionApprove.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `PATCH` `/api/v2/evidence_submissions/{evidence_submission_id}/replace`
   * @tag Evidence Submissions
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Evidence-Submissions/paths/~1api~1v2~1evidence_submissions~1%7Bevidence_submission_id%7D~1replace/patch)
   */
  public async updateEvidenceSubmissionReplace(
    params: { evidence_submission_id: string; include?: string },
    payload?: { data: Record<string, any> },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['updateEvidenceSubmissionReplace']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['updateEvidenceSubmissionReplace']>>
    >(this.#client.updateEvidenceSubmissionReplace.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/evidence_submissions/{evidence_submission_id}/qualitative_checks`
   * @tag Qualitative Checks
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Qualitative-Checks/paths/~1api~1v2~1evidence_submissions~1%7Bevidence_submission_id%7D~1qualitative_checks/get)
   */
  public async listEvidenceSubmissionQualitativeCheck(
    params: { evidence_submission_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listEvidenceSubmissionQualitativeCheck']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listEvidenceSubmissionQualitativeCheck']>>
    >(
      this.#client.listEvidenceSubmissionQualitativeCheck.bind(
        this.#client,
        params,
        undefined,
        config
      )
    )
  }

  /**
   * Operation Method for `GET` `/api/v2/qualitative_checks/{qualitative_check_id}`
   * @tag Qualitative Checks
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Qualitative-Checks/paths/~1api~1v2~1qualitative_checks~1%7Bqualitative_check_id%7D/get)
   */
  public async getQualitativeCheck(
    params: { qualitative_check_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getQualitativeCheck']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getQualitativeCheck']>>
    >(this.#client.getQualitativeCheck.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/evidence_submissions/{evidence_submission_id}/rejection_reasons`
   * @tag Rejection Reasons
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Rejection-Reasons/paths/~1api~1v2~1evidence_submissions~1%7Bevidence_submission_id%7D~1rejection_reasons/get)
   */
  public async listEvidenceSubmissionRejectionReason(
    params: { evidence_submission_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listEvidenceSubmissionRejectionReason']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listEvidenceSubmissionRejectionReason']>>
    >(
      this.#client.listEvidenceSubmissionRejectionReason.bind(
        this.#client,
        params,
        undefined,
        config
      )
    )
  }

  /**
   * Operation Method for `GET` `/api/v2/rejection_reasons/{rejection_reason_id}`
   * @tag Rejection Reasons
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Rejection-Reasons/paths/~1api~1v2~1rejection_reasons~1%7Brejection_reason_id%7D/get)
   */
  public async getRejectionReason(
    params: { rejection_reason_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getRejectionReason']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getRejectionReason']>>
    >(this.#client.getRejectionReason.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/evidence_submissions/{evidence_submission_id}/documents`
   * @tag Documents
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Documents/paths/~1api~1v2~1evidence_submissions~1%7Bevidence_submission_id%7D~1documents/get)
   */
  public async listEvidenceSubmissionDocument(
    params: { evidence_submission_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listEvidenceSubmissionDocument']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listEvidenceSubmissionDocument']>>
    >(this.#client.listEvidenceSubmissionDocument.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `POST` `/api/v2/evidence_submissions/{evidence_submission_id}/documents`
   * @tag Document Uploads
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Document-Uploads/paths/~1api~1v2~1evidence_submissions~1%7Bevidence_submission_id%7D~1documents/post)
   */
  public async createEvidenceSubmissionDocument(
    params: { evidence_submission_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['createEvidenceSubmissionDocument']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['createEvidenceSubmissionDocument']>>
    >(this.#client.createEvidenceSubmissionDocument.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/documents/{document_id}`
   * @tag Documents
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Documents/paths/~1api~1v2~1documents~1%7Bdocument_id%7D/get)
   */
  public async getDocument(
    params: { document_id: string; include?: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getDocument']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getDocument']>>
    >(this.#client.getDocument.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/documents/{document_id}/document_tokens`
   * @tag Document Tokens
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Document-Tokens/paths/~1api~1v2~1documents~1%7Bdocument_id%7D~1document_tokens/get)
   */
  public async listDocumentDocumentToken(
    params: { document_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listDocumentDocumentToken']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listDocumentDocumentToken']>>
    >(this.#client.listDocumentDocumentToken.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `POST` `/api/v2/documents/{document_id}/document_tokens`
   * @tag Document Tokens
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Document-Tokens/paths/~1api~1v2~1documents~1%7Bdocument_id%7D~1document_tokens/post)
   */
  public async createDocumentDocumentToken(
    params: { document_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['createDocumentDocumentToken']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['createDocumentDocumentToken']>>
    >(this.#client.createDocumentDocumentToken.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/document_tokens/{document_token_id}`
   * @tag Document Tokens
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Document-Tokens/paths/~1api~1v2~1document_tokens~1%7Bdocument_token_id%7D/get)
   */
  public async getDocumentToken(
    params: { document_token_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getDocumentToken']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getDocumentToken']>>
    >(this.#client.getDocumentToken.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/document_file_downloads/{document_token_id}`
   * @tag Download Documents
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Download-Documents/paths/~1api~1v2~1document_file_downloads~1%7Bdocument_token_id%7D/get)
   */
  public async getDocumentFileDownload(
    params: { document_token_id: Record<string, any> },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getDocumentFileDownload']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getDocumentFileDownload']>>
    >(this.#client.getDocumentFileDownload.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/forms/{form_id}`
   * @tag Forms
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Forms/paths/~1api~1v2~1forms~1%7Bform_id%7D/get)
   */
  public async getForm(
    params: { form_id: string; include?: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getForm']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getForm']>>
    >(this.#client.getForm.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/forms/{form_id}/form_data`
   * @tag Form Data
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Form-Data/paths/~1api~1v2~1forms~1%7Bform_id%7D~1form_data/get)
   */
  public async getFormFormData(
    params: { form_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getFormFormData']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getFormFormData']>>
    >(this.#client.getFormFormData.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `POST` `/api/v2/forms/{form_id}/form_data`
   * @tag Form Data
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Form-Data/paths/~1api~1v2~1forms~1%7Bform_id%7D~1form_data/post)
   */
  public async createForm(
    params: { form_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['createForm']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['createForm']>>
    >(this.#client.createForm.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/form_data/{form_datum_id}`
   * @tag Form Data
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Form-Data/paths/~1api~1v2~1form_data~1%7Bform_datum_id%7D/get)
   */
  public async getFormDataById(
    params: { form_datum_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getFormDataById']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getFormDataById']>>
    >(this.#client.getFormDataById.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/kyc_applications/{kyc_application_id}/related_entities`
   * @tag Related Entities
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Related-Entities/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D~1related_entities/get)
   */
  public async listKycApplicationRelatedEntitie(
    params: { kyc_application_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listKycApplicationRelatedEntitie']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listKycApplicationRelatedEntitie']>>
    >(this.#client.listKycApplicationRelatedEntitie.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `POST` `/api/v2/kyc_applications/{kyc_application_id}/related_entities`
   * @tag Related Entities
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Related-Entities/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D~1related_entities/post)
   */
  public async createKycApplicationRelatedEntitie(
    params: { kyc_application_id: string },
    payload?: { data: Record<string, any> },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['createKycApplicationRelatedEntitie']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['createKycApplicationRelatedEntitie']>>
    >(this.#client.createKycApplicationRelatedEntitie.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/related_entities/{related_entity_id}`
   * @tag Related Entities
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Related-Entities/paths/~1api~1v2~1related_entities~1%7Brelated_entity_id%7D/get)
   */
  public async getRelatedEntitie(
    params: { related_entity_id: string; include?: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getRelatedEntitie']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getRelatedEntitie']>>
    >(this.#client.getRelatedEntitie.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `DELETE` `/api/v2/related_entities/{related_entity_id}`
   * @tag Related Entities
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Related-Entities/paths/~1api~1v2~1related_entities~1%7Brelated_entity_id%7D/delete)
   */
  public async deleteRelatedEntitie(
    params: { related_entity_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['deleteRelatedEntitie']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['deleteRelatedEntitie']>>
    >(this.#client.deleteRelatedEntitie.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/kyc_applications/{kyc_application_id}/pdf_report_summaries`
   * @tag PDF Report Summaries
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/PDF-Report-Summaries/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D~1pdf_report_summaries/get)
   */
  public async listKycApplicationPdfReportSummarie(
    params: { kyc_application_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listKycApplicationPdfReportSummarie']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listKycApplicationPdfReportSummarie']>>
    >(
      this.#client.listKycApplicationPdfReportSummarie.bind(this.#client, params, undefined, config)
    )
  }

  /**
   * Operation Method for `POST` `/api/v2/kyc_applications/{kyc_application_id}/pdf_report_summaries`
   * @tag PDF Report Summaries
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/PDF-Report-Summaries/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D~1pdf_report_summaries/post)
   */
  public async createKycApplicationPdfReportSummarie(
    params: { kyc_application_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['createKycApplicationPdfReportSummarie']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['createKycApplicationPdfReportSummarie']>>
    >(
      this.#client.createKycApplicationPdfReportSummarie.bind(this.#client, params, payload, config)
    )
  }

  /**
   * Operation Method for `GET` `/api/v2/pdf_report_summaries/{pdf_report_summary_id}/`
   * @tag PDF Report Summaries
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/PDF-Report-Summaries/paths/~1api~1v2~1pdf_report_summaries~1%7Bpdf_report_summary_id%7D~1/get)
   */
  public async getPdfReportSummarie(
    params: { pdf_report_summary_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getPdfReportSummarie']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getPdfReportSummarie']>>
    >(this.#client.getPdfReportSummarie.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/pdf_report_summaries/{pdf_summary_report_id}/download`
   * @tag PDF Report Summaries
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/PDF-Report-Summaries/paths/~1api~1v2~1pdf_report_summaries~1%7Bpdf_summary_report_id%7D~1download/get)
   */
  public async getPdfReportSummarieDownload(
    params: { pdf_report_summary_id: string; pdf_summary_report_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getPdfReportSummarieDownload']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getPdfReportSummarieDownload']>>
    >(this.#client.getPdfReportSummarieDownload.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/users`
   * @tag Users
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Users/paths/~1api~1v2~1users/get)
   */
  public async listUser(
    params: { page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listUser']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listUser']>>
    >(this.#client.listUser.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/users/{user_id}`
   * @tag Users
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Users/paths/~1api~1v2~1users~1%7Buser_id%7D/get)
   */
  public async getUser(
    params: { user_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getUser']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getUser']>>
    >(this.#client.getUser.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/risk_rating_evaluations/{risk_rating_evaluation_id}`
   * @tag Risk Rating Evaluations
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Risk-Rating-Evaluations/paths/~1api~1v2~1risk_rating_evaluations~1%7Brisk_rating_evaluation_id%7D/get)
   */
  public async getRiskRatingEvaluation(
    params: { risk_rating_evaluation_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getRiskRatingEvaluation']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getRiskRatingEvaluation']>>
    >(this.#client.getRiskRatingEvaluation.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/kyc_applications/{kyc_application_id}/risk_rating_evaluations`
   * @tag Risk Rating Evaluations
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Risk-Rating-Evaluations/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D~1risk_rating_evaluations/get)
   */
  public async listKycApplicationRiskRatingEvaluation(
    params: { kyc_application_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listKycApplicationRiskRatingEvaluation']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listKycApplicationRiskRatingEvaluation']>>
    >(
      this.#client.listKycApplicationRiskRatingEvaluation.bind(
        this.#client,
        params,
        undefined,
        config
      )
    )
  }

  /**
   * Operation Method for `GET` `/api/v2/kyc_applications/{kyc_application_id}/risk_ratings`
   * @tag Risk Ratings
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Risk-Ratings/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D~1risk_ratings/get)
   */
  public async listKycApplicationRiskRating(
    params: { kyc_application_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listKycApplicationRiskRating']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listKycApplicationRiskRating']>>
    >(this.#client.listKycApplicationRiskRating.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `POST` `/api/v2/kyc_applications/{kyc_application_id}/risk_ratings`
   * @tag Risk Ratings
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Risk-Ratings/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D~1risk_ratings/post)
   */
  public async createKycApplicationRiskRating(
    params: { kyc_application_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['createKycApplicationRiskRating']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['createKycApplicationRiskRating']>>
    >(this.#client.createKycApplicationRiskRating.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/kyc_applications/{kyc_application_id}/risk_ratings/{evaluation_id}`
   * @tag Risk Ratings
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Risk-Ratings/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D~1risk_ratings~1%7Bevaluation_id%7D/get)
   */
  public async getKycApplicationRiskRating(
    params: { evaluation_id: string; kyc_application_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getKycApplicationRiskRating']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getKycApplicationRiskRating']>>
    >(this.#client.getKycApplicationRiskRating.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `PATCH` `/api/v2/kyc_applications/{kyc_application_id}/risk_ratings/{evaluation_id}`
   * @tag Risk Ratings
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Risk-Ratings/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D~1risk_ratings~1%7Bevaluation_id%7D/patch)
   */
  public async updateKycApplicationRiskRating(
    params: { kyc_application_id: string; evaluation_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['updateKycApplicationRiskRating']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['updateKycApplicationRiskRating']>>
    >(this.#client.updateKycApplicationRiskRating.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/kyc_applications/{kyc_application_id}/risk_ratings/questions`
   * @tag Risk Ratings
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Risk-Ratings/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D~1risk_ratings~1questions/get)
   */
  public async listKycApplicationRiskRatingsQuestion(
    params: { kyc_application_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listKycApplicationRiskRatingsQuestion']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listKycApplicationRiskRatingsQuestion']>>
    >(
      this.#client.listKycApplicationRiskRatingsQuestion.bind(
        this.#client,
        params,
        undefined,
        config
      )
    )
  }

  /**
   * Operation Method for `GET` `/api/v2/bank_account_schemas/{kyc_application_id}`
   * @tag Bank Account Verification
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Bank-Account-Verification/paths/~1api~1v2~1bank_account_schemas~1%7Bkyc_application_id%7D/get)
   */
  public async getBankAccountSchema(
    params: { kyc_application_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getBankAccountSchema']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getBankAccountSchema']>>
    >(this.#client.getBankAccountSchema.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/kyc_applications/{kyc_application_id}/bank_accounts`
   * @tag Bank Account Verification
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Bank-Account-Verification/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D~1bank_accounts/get)
   */
  public async listKycApplicationBankAccount(
    params: { kyc_application_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listKycApplicationBankAccount']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listKycApplicationBankAccount']>>
    >(this.#client.listKycApplicationBankAccount.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `POST` `/api/v2/kyc_applications/{kyc_application_id}/bank_accounts`
   * @tag Bank Account Verification
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Bank-Account-Verification/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D~1bank_accounts/post)
   */
  public async createKycApplicationBankAccount(
    params: { kyc_application_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['createKycApplicationBankAccount']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['createKycApplicationBankAccount']>>
    >(this.#client.createKycApplicationBankAccount.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/bank_accounts/{bank_account_id}`
   * @tag Bank Account Verification
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Bank-Account-Verification/paths/~1api~1v2~1bank_accounts~1%7Bbank_account_id%7D/get)
   */
  public async getBankAccount(
    params: { bank_account_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getBankAccount']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getBankAccount']>>
    >(this.#client.getBankAccount.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/bank_account_verifications/{bank_account_verification_id}`
   * @tag Bank Account Verification
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Bank-Account-Verification/paths/~1api~1v2~1bank_account_verifications~1%7Bbank_account_verification_id%7D/get)
   */
  public async getBankAccountVerification(
    params: { bank_account_verification_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getBankAccountVerification']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getBankAccountVerification']>>
    >(this.#client.getBankAccountVerification.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `PATCH` `/api/v2/bank_account_verifications/{bank_account_verification_id}`
   * @tag Bank Account Verification
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Bank-Account-Verification/paths/~1api~1v2~1bank_account_verifications~1%7Bbank_account_verification_id%7D/patch)
   */
  public async updateBankAccountVerification(
    params: { bank_account_verification_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['updateBankAccountVerification']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['updateBankAccountVerification']>>
    >(this.#client.updateBankAccountVerification.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/bank_accounts/{bank_account_id}/bank_account_verifications`
   * @tag Bank Account Verification
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Bank-Account-Verification/paths/~1api~1v2~1bank_accounts~1%7Bbank_account_id%7D~1bank_account_verifications/get)
   */
  public async listBankAccountBankAccountVerification(
    params: { bank_account_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listBankAccountBankAccountVerification']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listBankAccountBankAccountVerification']>>
    >(
      this.#client.listBankAccountBankAccountVerification.bind(
        this.#client,
        params,
        undefined,
        config
      )
    )
  }

  /**
   * Operation Method for `GET` `/api/v2/kyc_applications/{kyc_application_id}/consumer_report_queries`
   * @tag Consumer Report Query
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Consumer-Report-Query/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D~1consumer_report_queries/get)
   */
  public async listKycApplicationConsumerReportQuerie(
    params: { kyc_application_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listKycApplicationConsumerReportQuerie']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listKycApplicationConsumerReportQuerie']>>
    >(
      this.#client.listKycApplicationConsumerReportQuerie.bind(
        this.#client,
        params,
        undefined,
        config
      )
    )
  }

  /**
   * Operation Method for `GET` `/api/v2/consumer_report_queries/{id}`
   * @tag Consumer Report Query
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Consumer-Report-Query/paths/~1api~1v2~1consumer_report_queries~1%7Bid%7D/get)
   */
  public async getConsumerReportQuerie(
    params: { id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getConsumerReportQuerie']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getConsumerReportQuerie']>>
    >(this.#client.getConsumerReportQuerie.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/consumer_report_queries/{id}/download`
   * @tag Consumer Report Query
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Consumer-Report-Query/paths/~1api~1v2~1consumer_report_queries~1%7Bid%7D~1download/get)
   */
  public async getConsumerReportQuerieDownload(
    params: { id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getConsumerReportQuerieDownload']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getConsumerReportQuerieDownload']>>
    >(this.#client.getConsumerReportQuerieDownload.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `POST` `/api/v2/onboarding_invitations`
   * @tag Onboarding Invitation
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Onboarding-Invitation/paths/~1api~1v2~1onboarding_invitations/post)
   */
  public async createOnboardingInvitation(
    params: {},
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['createOnboardingInvitation']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['createOnboardingInvitation']>>
    >(this.#client.createOnboardingInvitation.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `PATCH` `/api/v2/onboarding_invitations/{kyc_application_id}`
   * @tag Onboarding Invitation
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Onboarding-Invitation/paths/~1api~1v2~1onboarding_invitations~1%7Bkyc_application_id%7D/patch)
   */
  public async updateOnboardingInvitation(
    params: { kyc_application_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['updateOnboardingInvitation']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['updateOnboardingInvitation']>>
    >(this.#client.updateOnboardingInvitation.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/forms/{form_id}/form_pdf_exports`
   * @tag Form PDF Exports
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Form-PDF-Exports/paths/~1api~1v2~1forms~1%7Bform_id%7D~1form_pdf_exports/get)
   */
  public async listFormFormPdfExport(
    params: { form_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listFormFormPdfExport']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listFormFormPdfExport']>>
    >(this.#client.listFormFormPdfExport.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `POST` `/api/v2/forms/{form_id}/form_pdf_exports`
   * @tag Form PDF Exports
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Form-PDF-Exports/paths/~1api~1v2~1forms~1%7Bform_id%7D~1form_pdf_exports/post)
   */
  public async createFormFormPdfExport(
    params: { form_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['createFormFormPdfExport']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['createFormFormPdfExport']>>
    >(this.#client.createFormFormPdfExport.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/form_pdf_exports/{form_pdf_export_id}/`
   * @tag Form PDF Exports
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Form-PDF-Exports/paths/~1api~1v2~1form_pdf_exports~1%7Bform_pdf_export_id%7D~1/get)
   */
  public async getFormPdfExport(
    params: { form_pdf_export_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getFormPdfExport']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getFormPdfExport']>>
    >(this.#client.getFormPdfExport.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/form_pdf_exports/{form_pdf_export_id}/download`
   * @tag Form PDF Exports
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Form-PDF-Exports/paths/~1api~1v2~1form_pdf_exports~1%7Bform_pdf_export_id%7D~1download/get)
   */
  public async getFormPdfExportDownload(
    params: { form_pdf_export_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getFormPdfExportDownload']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getFormPdfExportDownload']>>
    >(this.#client.getFormPdfExportDownload.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/kyc_entity_templates/{kyc_entity_template_id}/designation_templates`
   * @tag Related Entity Designations
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Related-Entity-Designations/paths/~1api~1v2~1kyc_entity_templates~1%7Bkyc_entity_template_id%7D~1designation_templates/get)
   */
  public async listKycEntityTemplateDesignationTemplate(
    params: { kyc_entity_template_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listKycEntityTemplateDesignationTemplate']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listKycEntityTemplateDesignationTemplate']>>
    >(
      this.#client.listKycEntityTemplateDesignationTemplate.bind(
        this.#client,
        params,
        undefined,
        config
      )
    )
  }

  /**
   * Operation Method for `GET` `/api/v2/related_entity_designations/{related_entity_designation_id}`
   * @tag Related Entity Designations
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Related-Entity-Designations/paths/~1api~1v2~1related_entity_designations~1%7Brelated_entity_designation_id%7D/get)
   */
  public async getRelatedEntityDesignation(
    params: { related_entity_designation_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getRelatedEntityDesignation']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getRelatedEntityDesignation']>>
    >(this.#client.getRelatedEntityDesignation.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `DELETE` `/api/v2/related_entity_designations/{related_entity_designation_id}`
   * @tag Related Entity Designations
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Related-Entity-Designations/paths/~1api~1v2~1related_entity_designations~1%7Brelated_entity_designation_id%7D/delete)
   */
  public async deleteRelatedEntityDesignation(
    params: { related_entity_designation_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['deleteRelatedEntityDesignation']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['deleteRelatedEntityDesignation']>>
    >(this.#client.deleteRelatedEntityDesignation.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/related_entities/{related_entity_id}/related_entity_designations`
   * @tag Related Entity Designations
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Related-Entity-Designations/paths/~1api~1v2~1related_entities~1%7Brelated_entity_id%7D~1related_entity_designations/get)
   */
  public async listRelatedEntitieRelatedEntityDesignation(
    params: { related_entity_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listRelatedEntitieRelatedEntityDesignation']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listRelatedEntitieRelatedEntityDesignation']>>
    >(
      this.#client.listRelatedEntitieRelatedEntityDesignation.bind(
        this.#client,
        params,
        undefined,
        config
      )
    )
  }

  /**
   * Operation Method for `POST` `/api/v2/related_entities/{related_entity_id}/related_entity_designations`
   * @tag Related Entity Designations
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Related-Entity-Designations/paths/~1api~1v2~1related_entities~1%7Brelated_entity_id%7D~1related_entity_designations/post)
   */
  public async createRelatedEntitieRelatedEntityDesignation(
    params: { related_entity_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['createRelatedEntitieRelatedEntityDesignation']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['createRelatedEntitieRelatedEntityDesignation']>>
    >(
      this.#client.createRelatedEntitieRelatedEntityDesignation.bind(
        this.#client,
        params,
        payload,
        config
      )
    )
  }

  /**
   * Operation Method for `GET` `/api/v2/products/{product_id}/product_product_addons`
   * @tag Product Addons
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Product-Addons/paths/~1api~1v2~1products~1%7Bproduct_id%7D~1product_product_addons/get)
   */
  public async listProductProductProductAddon(
    params: { product_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listProductProductProductAddon']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listProductProductProductAddon']>>
    >(this.#client.listProductProductProductAddon.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/account_applications/{account_application_id}/account_addons`
   * @tag Account Addons
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Account-Addons/paths/~1api~1v2~1account_applications~1%7Baccount_application_id%7D~1account_addons/get)
   */
  public async listAccountApplicationAccountAddon(
    params: { account_application_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listAccountApplicationAccountAddon']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listAccountApplicationAccountAddon']>>
    >(this.#client.listAccountApplicationAccountAddon.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/account_addons/{account_addon_id}`
   * @tag Account Addons
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Account-Addons/paths/~1api~1v2~1account_addons~1%7Baccount_addon_id%7D/get)
   */
  public async getAccountAddon(
    params: { account_addon_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getAccountAddon']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getAccountAddon']>>
    >(this.#client.getAccountAddon.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/products/{product_id}/disclosures`
   * @tag Product Disclosures
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Product-Disclosures/paths/~1api~1v2~1products~1%7Bproduct_id%7D~1disclosures/get)
   */
  public async listProductDisclosure(
    params: { product_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listProductDisclosure']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listProductDisclosure']>>
    >(this.#client.listProductDisclosure.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/disclosures`
   * @tag Disclosures
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Disclosures/paths/~1api~1v2~1disclosures/get)
   */
  public async listDisclosure(
    params: { page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listDisclosure']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listDisclosure']>>
    >(this.#client.listDisclosure.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/disclosures/{disclosure_id}`
   * @tag Disclosures
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Disclosures/paths/~1api~1v2~1disclosures~1%7Bdisclosure_id%7D/get)
   */
  public async getDisclosure(
    params: { disclosure_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getDisclosure']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getDisclosure']>>
    >(this.#client.getDisclosure.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/products`
   * @tag Products
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Products/paths/~1api~1v2~1products/get)
   */
  public async listProduct(
    params: { page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listProduct']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listProduct']>>
    >(this.#client.listProduct.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/products/{id}`
   * @tag Products
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Products/paths/~1api~1v2~1products~1%7Bid%7D/get)
   */
  public async getProduct(
    params: { id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getProduct']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getProduct']>>
    >(this.#client.getProduct.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/kyc_entity_templates/{kyc_entity_template_id}/products`
   * @tag KycEntityTemplates Products
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KycEntityTemplates-Products/paths/~1api~1v2~1kyc_entity_templates~1%7Bkyc_entity_template_id%7D~1products/get)
   */
  public async listKycEntityTemplateProduct(
    params: { kyc_entity_template_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listKycEntityTemplateProduct']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listKycEntityTemplateProduct']>>
    >(this.#client.listKycEntityTemplateProduct.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/account_applications/{id}/account_summary_pdf`
   * @tag Account Application Pdf Summary
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Account-Application-Pdf-Summary/paths/~1api~1v2~1account_applications~1%7Bid%7D~1account_summary_pdf/get)
   */
  public async getAccountApplicationAccountSummaryPdf(
    params: { id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getAccountApplicationAccountSummaryPdf']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getAccountApplicationAccountSummaryPdf']>>
    >(
      this.#client.getAccountApplicationAccountSummaryPdf.bind(
        this.#client,
        params,
        undefined,
        config
      )
    )
  }

  /**
   * Operation Method for `GET` `/api/v2/kyc_entity_templates/{kyc_entity_template_id}/disclosures`
   * @tag KycEntityTemplates Disclosures
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/KycEntityTemplates-Disclosures/paths/~1api~1v2~1kyc_entity_templates~1%7Bkyc_entity_template_id%7D~1disclosures/get)
   */
  public async listKycEntityTemplateDisclosure(
    params: { kyc_entity_template_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listKycEntityTemplateDisclosure']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listKycEntityTemplateDisclosure']>>
    >(this.#client.listKycEntityTemplateDisclosure.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/account_applications/{account_application_id}/data_requirements`
   * @tag Data Requirements
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Data-Requirements/paths/~1api~1v2~1account_applications~1%7Baccount_application_id%7D~1data_requirements/get)
   */
  public async listAccountApplicationDataRequirement(
    params: { account_application_id: string; forms?: boolean; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listAccountApplicationDataRequirement']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listAccountApplicationDataRequirement']>>
    >(
      this.#client.listAccountApplicationDataRequirement.bind(
        this.#client,
        params,
        undefined,
        config
      )
    )
  }

  /**
   * Operation Method for `GET` `/api/v2/product_product_addons/disclosures`
   * @tag Product Product Addon Disclosures
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Product-Product-Addon-Disclosures/paths/~1api~1v2~1product_product_addons~1disclosures/get)
   */
  public async listProductProductAddonsDisclosure(
    params: { product_product_addon_ids?: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listProductProductAddonsDisclosure']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listProductProductAddonsDisclosure']>>
    >(this.#client.listProductProductAddonsDisclosure.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `GET` `/api/v2/account_applications/{id}/account_summary`
   * @tag Account Application Summary
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Account-Application-Summary/paths/~1api~1v2~1account_applications~1%7Bid%7D~1account_summary/get)
   */
  public async getAccountApplicationAccountSummary(
    params: { id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getAccountApplicationAccountSummary']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getAccountApplicationAccountSummary']>>
    >(
      this.#client.getAccountApplicationAccountSummary.bind(this.#client, params, undefined, config)
    )
  }

  /**
   * Operation Method for `GET` `/api/v2/kyc_applications/{kyc_application_id}/account_applications`
   * @tag Account Applications
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Account-Applications/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D~1account_applications/get)
   */
  public async listKycApplicationAccountApplication(
    params: { kyc_application_id: string; page?: number; per_page?: number },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['listKycApplicationAccountApplication']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['listKycApplicationAccountApplication']>>
    >(
      this.#client.listKycApplicationAccountApplication.bind(
        this.#client,
        params,
        undefined,
        config
      )
    )
  }

  /**
   * Operation Method for `POST` `/api/v2/kyc_applications/{kyc_application_id}/account_applications`
   * @tag Account Applications
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Account-Applications/paths/~1api~1v2~1kyc_applications~1%7Bkyc_application_id%7D~1account_applications/post)
   */
  public async createKycApplicationAccountApplication(
    params: { kyc_application_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['createKycApplicationAccountApplication']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['createKycApplicationAccountApplication']>>
    >(
      this.#client.createKycApplicationAccountApplication.bind(
        this.#client,
        params,
        payload,
        config
      )
    )
  }

  /**
   * Operation Method for `GET` `/api/v2/account_applications/{account_application_id}`
   * @tag Account Applications
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Account-Applications/paths/~1api~1v2~1account_applications~1%7Baccount_application_id%7D/get)
   */
  public async getAccountApplication(
    params: { account_application_id: string },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['getAccountApplication']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['getAccountApplication']>>
    >(this.#client.getAccountApplication.bind(this.#client, params, undefined, config))
  }

  /**
   * Operation Method for `PATCH` `/api/v2/account_applications/{account_application_id}`
   * @tag Account Applications
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Account-Applications/paths/~1api~1v2~1account_applications~1%7Baccount_application_id%7D/patch)
   */
  public async updateAccountApplication(
    params: { account_application_id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['updateAccountApplication']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['updateAccountApplication']>>
    >(this.#client.updateAccountApplication.bind(this.#client, params, payload, config))
  }

  /**
   * Operation Method for `PATCH` `/api/v2/account_applications/{id}/disclosure_acceptances`
   * @tag Disclosure Acceptances
   * @see [DocFox Documentation](https://www.docfoxapp.com/api/v2/documentation#tag/Disclosure-Acceptances/paths/~1api~1v2~1account_applications~1%7Bid%7D~1disclosure_acceptances/patch)
   */
  public async updateAccountApplicationDisclosureAcceptances(
    params: { id: string },
    payload?: { data: any },
    config?: AxiosRequestConfig<any>
  ): ReturnType<DocFoxAxiosClient['updateAccountApplicationDisclosureAcceptances']> {
    return await this.#doClientRequestWithAuthenticationRetry<
      Awaited<ReturnType<DocFoxAxiosClient['updateAccountApplicationDisclosureAcceptances']>>
    >(
      this.#client.updateAccountApplicationDisclosureAcceptances.bind(
        this.#client,
        params,
        payload,
        config
      )
    )
  }

  public static error(message?: string, response?: AxiosResponse) {
    return new DocFoxError(message, response)
  }

  public static errorFromAuth(message?: string, response?: AxiosResponse) {
    return new DocFoxAuthenticationError(message, response)
  }
}
